@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #SPDX-License-Identifier: GPL-3.0-or-later
-#myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+#myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
 #https://github.com/jcorporation/mympd
 
 #exit on error
@@ -36,24 +36,6 @@ echo_warn() {
   printf "\e[m"
 }
 
-#clang tidy options
-CLANG_TIDY_CHECKS="*"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-altera-id-dependent-backward-branch"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-altera-unroll-loops"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-altera-struct-pack-align,-clang-analyzer-optin.performance.Padding"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-bugprone-easily-swappable-parameters"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-bugprone-signal-handler,-cert-sig30-c"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-bugprone-assignment-in-if-condition"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-clang-diagnostic-invalid-command-line-argument"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-concurrency-mt-unsafe"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-cppcoreguidelines*"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-hicpp-*"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-llvmlibc-restrict-system-libc-headers"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-readability-identifier-length"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-readability-function-cognitive-complexity,-google-readability-function-size,-readability-function-size"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-readability-magic-numbers"
-CLANG_TIDY_CHECKS="$CLANG_TIDY_CHECKS,-readability-non-const-parameter"
-
 #save script path and change to it
 STARTPATH=$(dirname "$(realpath "$0")")
 cd "$STARTPATH" || exit 1
@@ -63,7 +45,17 @@ umask 0022
 
 #get myMPD version
 VERSION=$(grep "  VERSION" CMakeLists.txt | sed 's/  VERSION //')
-COPYRIGHT="myMPD ${VERSION} | (c) 2018-2022 Juergen Mang <mail@jcgames.de> | SPDX-License-Identifier: GPL-3.0-or-later | https://github.com/jcorporation/mympd"
+COPYRIGHT="myMPD ${VERSION} | (c) 2018-2024 Juergen Mang <mail@jcgames.de> | SPDX-License-Identifier: GPL-3.0-or-later | https://github.com/jcorporation/mympd"
+
+# Minify JavaScript only for master branch
+if [ -z "${MYMPD_MINIFY_JS+x}" ]
+then
+  MYMPD_MINIFY_JS="1"
+  if [ -f .git/HEAD ] && ! grep -q "master" .git/HEAD
+  then
+    MYMPD_MINIFY_JS="0"
+  fi
+fi
 
 #check for command
 check_cmd() {
@@ -106,7 +98,8 @@ setversion() {
 
   for F in contrib/packaging/alpine/APKBUILD contrib/packaging/arch/PKGBUILD \
       contrib/packaging/rpm/mympd.spec contrib/packaging/debian/changelog \
-      contrib/packaging/openwrt/Makefile contrib/man/mympd.1 contrib/man/mympd-script.1
+      contrib/packaging/openwrt/Makefile contrib/man/mympd.1 contrib/man/mympd-config.1 \
+      contrib/man/mympd-script.1 contrib/packaging/freebsd/multimedia/mympd/Makefile
   do
     echo "$F"
     sed -e "s/__VERSION__/${VERSION}/g" -e "s/__DATE_F1__/$DATE_F1/g" -e "s/__DATE_F2__/$DATE_F2/g" \
@@ -120,8 +113,11 @@ setversion() {
       "contrib/packaging/gentoo/media-sound/mympd/mympd-${VERSION}.ebuild"
   fi
 
-  echo "const myMPDversion = '${VERSION}';" > htdocs/js/version.js
   printf "%s" "${VERSION}" > docs/_includes/version
+
+  BUILD=$(git log --format="%H" -n 1)
+  echo "const myMPDversion = '${VERSION}';" > htdocs/js/version.js
+  echo "const myMPDbuild = '${BUILD}';" >> htdocs/js/version.js
 }
 
 minify() {
@@ -143,11 +139,16 @@ minify() {
   elif [ "$TYPE" = "js" ]
   then
     #shellcheck disable=SC2016
-    if ! perl -pe 's/^\s*//gm; s/^\s*\/?\*.*$//g; s/^\/\/.+$//g; s/^logDebug\(.*$//g; s/\/\*debug\*\/.*$//g; s/\s*$//gm;' "$SRC" > "${DST}.tmp"
+    if [ "$MYMPD_MINIFY_JS" = "0" ]
     then
-      rm -f "${DST}.tmp"
-      echo_error "Error minifying $SRC"
-      exit 1
+      cp "$SRC" "${DST}.tmp"
+    else
+      if ! perl -pe 's/^\s*//gm; s/^\s*\/?\*.*$//g; s/^\/\/.+$//g; s/^logDebug\(.*$//g; s/\/\*debug\*\/.*$//g; s/\s*$//gm;' "$SRC" > "${DST}.tmp"
+      then
+        rm -f "${DST}.tmp"
+        echo_error "Error minifying $SRC"
+        exit 1
+      fi
     fi
   elif [ "$TYPE" = "json" ]
   then
@@ -188,9 +189,13 @@ createassets() {
   install -d "$MYMPD_BUILDDIR/htdocs/assets/i18n"
 
   #Create translation phrases file
-  createi18n "$MYMPD_BUILDDIR" 2>/dev/null
+  check_phrases
+  createi18n 2>/dev/null
   minify js "$MYMPD_BUILDDIR/htdocs/js/i18n.js" "$MYMPD_BUILDDIR/htdocs/js/i18n.min.js"
 
+  #Create defines
+  create_js_defines
+
   echo "Minifying javascript"
   JSSRCFILES=""
   #shellcheck disable=SC2013
@@ -215,7 +220,12 @@ createassets() {
 
   echo "Combining and compressing javascript"
   echo "//${COPYRIGHT}" > "$MYMPD_BUILDDIR/htdocs/js/copyright.min.js"
-  JSFILES="dist/bootstrap-native/bootstrap-native.min.js dist/long-press-event/long-press-event.min.js"
+  if [ "$MYMPD_MINIFY_JS" = "0" ]
+  then
+    JSFILES="dist/bootstrap-native/bootstrap-native.js dist/long-press-event/long-press-event.js"
+  else
+    JSFILES="dist/bootstrap-native/bootstrap-native.min.js dist/long-press-event/long-press-event.min.js"
+  fi
   JSFILES="$JSFILES $MYMPD_BUILDDIR/htdocs/js/*.min.js"
   for F in $JSFILES
   do
@@ -249,14 +259,6 @@ createassets() {
   cat $CSSFILES > "$MYMPD_BUILDDIR/htdocs/css/combined.css"
   $ZIP "$MYMPD_BUILDDIR/htdocs/css/combined.css"
 
-  echo "Compressing fonts"
-  FONTFILES="dist/material-icons/MaterialIcons-Regular.woff2 dist/material-icons/ligatures.json"
-  for FONT in $FONTFILES
-  do
-    DST=$(basename "${FONT}")
-    $ZIPCAT "$FONT" > "$MYMPD_BUILDDIR/htdocs/assets/${DST}.gz"
-  done
-
   echo "Compressing i18n json"
   jq -r "select(.missingPhrases < 100) | keys[]" "$STARTPATH/src/i18n/json/i18n.json" | grep -v "default" | \
     while read -r CODE
@@ -279,16 +281,60 @@ createassets() {
   echo "Copy images"
   cp -v htdocs/assets/*.png "$MYMPD_BUILDDIR/htdocs/assets/"
 
-  echo "Copy integrated lua libraries"
-  mkdir -p "$MYMPD_BUILDDIR/contrib/lualibs"
-  cp -v contrib/lualibs/*.lua "$MYMPD_BUILDDIR/contrib/lualibs/"
+  echo "Copy webfont"
+  cp -v dist/material-icons/MaterialIcons-Regular.woff2 "$MYMPD_BUILDDIR/htdocs/assets/"
+  $ZIPCAT dist/material-icons/ligatures.json > "$MYMPD_BUILDDIR/htdocs/assets/ligatures.json.gz"
+
+  [ -z "${MYMPD_ENABLE_LUA+x}" ] && MYMPD_ENABLE_LUA="ON"
+  if [ "${MYMPD_ENABLE_LUA}" = "on" ] || [ "${MYMPD_ENABLE_LUA}" = "ON" ]
+  then
+    lualibs
+  else
+    echo "Skip creation of lua libraries"
+  fi
+
   return 0
 }
 
+lualibs() {
+  [ -z "${MYMPD_ENABLE_MYGPIOD+x}" ] && MYMPD_ENABLE_MYGPIOD="OFF"
+  [ -z "${MYMPD_BUILDDIR+x}" ] && MYMPD_BUILDDIR="release"
+  echo "Copy integrated lua libraries"
+  mkdir -p "$MYMPD_BUILDDIR/contrib/lualibs"
+  cp -v contrib/lualibs/json.lua "$MYMPD_BUILDDIR/contrib/lualibs/"
+  cp -v contrib/lualibs/mympd/00-start.lua "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/10-mympd.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/20-http.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/30-execute.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  [ "$MYMPD_ENABLE_MYGPIOD" = "ON" ] && cat contrib/lualibs/mympd/40-mygpiod.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/50-util.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/60-caches.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  cat contrib/lualibs/mympd/99-end.lua >> "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  echo "Compiling lua libraries"
+  LUAC=$(command -v luac5.4 2> /dev/null || command -v luac5.3 2> /dev/null || command -v luac 2> /dev/null || true)
+  if [ -z "$LUAC" ]
+  then
+    echo_error "luac not found"
+    exit 1
+  fi
+  if ! $LUAC -s -o "$MYMPD_BUILDDIR/contrib/lualibs/mympd.luac" "$MYMPD_BUILDDIR/contrib/lualibs/mympd.lua"
+  then
+    exit 1
+  fi
+  if ! $LUAC -s -o "$MYMPD_BUILDDIR/contrib/lualibs/json.luac" "$MYMPD_BUILDDIR/contrib/lualibs/json.lua"
+  then
+    exit 1
+  fi
+}
+
 buildrelease() {
-  echo "Compiling myMPD v${VERSION}" 
-  cmake -B release -DCMAKE_INSTALL_PREFIX:PATH=/usr -DCMAKE_BUILD_TYPE=Release .
-  make -C release
+  BUILD_TYPE=$1
+  echo "Compiling myMPD v${VERSION}"
+  cmake -B release \
+    -DCMAKE_INSTALL_PREFIX:PATH=/usr \
+    -DCMAKE_BUILD_TYPE="$BUILD_TYPE" \
+    .
+  make -j4 -C release
 }
 
 addmympduser() {
@@ -330,7 +376,13 @@ installrelease() {
   cd release || exit 1
   [ -z "${DESTDIR+x}" ] && DESTDIR=""
   make install DESTDIR="$DESTDIR"
-  addmympduser
+  if [ ! -f /usr/lib/systemd/system/mympd.service ] &&
+     [ ! -f /usr/systemd/system/mympd.service ]
+  then
+    addmympduser
+  else
+    echo "Systemd found skipping mympd user creation"
+  fi
   echo "myMPD installed"
 }
 
@@ -343,8 +395,15 @@ copyassets() {
   cp -v "$STARTPATH/dist/long-press-event/long-press-event.js" "$STARTPATH/htdocs/js/long-press-event.js"
   cp -v "$STARTPATH/dist/material-icons/MaterialIcons-Regular.woff2" "$STARTPATH/htdocs/assets/MaterialIcons-Regular.woff2"
   cp -v "$STARTPATH/dist/material-icons/ligatures.json" "$STARTPATH/htdocs/assets/ligatures.json"
+
+  lualibs
+
+  #Create defines
+  create_js_defines
+
   #translation files
-  createi18n "$MYMPD_BUILDDIR"
+  check_phrases
+  createi18n
   cp -v "$MYMPD_BUILDDIR/htdocs/js/i18n.js" "$STARTPATH/htdocs/js/i18n.js"
   rm -fr "$STARTPATH/htdocs/assets/i18n/"
   install -d "$STARTPATH/htdocs/assets/i18n"
@@ -357,27 +416,41 @@ copyassets() {
 
 builddebug() {
   echo "Compiling myMPD v${VERSION}"
-  if [ "$ACTION" = "memcheck" ]
-  then
-    MYMPD_ENABLE_LIBASAN=ON
-  else
-    MYMPD_ENABLE_LIBASAN=OFF
-  fi
-  cmake -B debug -DCMAKE_INSTALL_PREFIX:PATH=/usr -DCMAKE_BUILD_TYPE=Debug \
-    -DCMAKE_EXPORT_COMPILE_COMMANDS=ON -DMYMPD_ENABLE_LIBASAN="$MYMPD_ENABLE_LIBASAN" \
+  CMAKE_SANITIZER_OPTIONS=""
+  case "$ACTION" in
+    asan)  CMAKE_SANITIZER_OPTIONS="-DMYMPD_ENABLE_ASAN=ON" ;;
+    tsan)  CMAKE_SANITIZER_OPTIONS="-DMYMPD_ENABLE_TSAN=ON" ;;
+    ubsan) CMAKE_SANITIZER_OPTIONS="-DMYMPD_ENABLE_UBSAN=ON" ;;
+  esac
+
+  cmake -B debug \
+    -DCMAKE_INSTALL_PREFIX:PATH=/usr \
+    -DCMAKE_BUILD_TYPE=Debug \
+    -DCMAKE_EXPORT_COMPILE_COMMANDS=ON \
+    $CMAKE_SANITIZER_OPTIONS \
     .
-  make -C debug VERBOSE=1
+  make -j4 -C debug VERBOSE=1
   echo "Linking compilation database"
   sed -e 's/\\t/ /g' -e 's/-Wformat-truncation//g' -e 's/-Wformat-overflow=2//g' -e 's/-fsanitize=bounds-strict//g' \
-    -e 's/-static-libasan//g' -e 's/-Wno-stringop-overread//g' -e 's/-fstack-clash-protection//g' \
+    -e 's/-Wno-stringop-overread//g' -e 's/-fstack-clash-protection//g' \
     debug/compile_commands.json > src/compile_commands.json
 }
 
 buildtest() {
   echo "Compiling and running unit tests"
-  cmake -B test/build -S test -DCMAKE_BUILD_TYPE=Debug
-  make -C test/build VERBOSE=1
-  ./test/build/test
+  cmake -B debug \
+    -DCMAKE_INSTALL_PREFIX:PATH=/usr \
+    -DCMAKE_BUILD_TYPE=Debug \
+    -DCMAKE_EXPORT_COMPILE_COMMANDS=ON \
+    -DMYMPD_ENABLE_ASAN=ON \
+    -DMYMPD_BUILD_TESTING=ON \
+    .
+  make -j4 -C debug
+  make -j4 -C debug test
+  echo "Linking compilation database"
+  sed -e 's/\\t/ /g' -e 's/-Wformat-truncation//g' -e 's/-Wformat-overflow=2//g' -e 's/-fsanitize=bounds-strict//g' \
+    -e 's/-Wno-stringop-overread//g' -e 's/-fstack-clash-protection//g' \
+    debug/compile_commands.json > test/compile_commands.json
 }
 
 cleanup() {
@@ -400,12 +473,21 @@ cleanup() {
   rm -rf htdocs/assets/i18n
 
   #generated documentation
+  rm -rf docs/_site
+  rm -rf docs/.jekyll-cache
   rm -rf docs/doxygen
   rm -rf docs/jsdoc
 
   #compilation database
   rm -f src/compile_commands.json
 
+  #caches
+  rm -fr src/.cache
+  rm -fr .cache
+
+  #node modules
+  rm -fr dist/bootstrap/node_modules
+
   #clang tidy
   rm -f clang-tidy.out
 }
@@ -485,15 +567,15 @@ check_file() {
   then
     echo "Running clang-tidy"
     rm -f clang-tidy.out
-    clang-tidy --checks="$CLANG_TIDY_CHECKS" \
-      "$FILE" > ../clang-tidy.out 2>/dev/null
-    grep -v -E "(/usr/include/|memset|memcpy|\^)" ../clang-tidy.out
+    clang-tidy --config-file="$STARTPATH/.clang-tidy" "$FILE" > ../clang-tidy.out 2>/dev/null
+    grep -v -E "(/usr/include/|memset|memcpy|_XOPEN_SOURCE|\^)" ../clang-tidy.out
   else
     echo_warn "clang-tidy not found"
   fi
 }
 
 check() {
+  check_phrases
   if ! check_docs
   then
     return 1
@@ -504,52 +586,6 @@ check() {
     return 1
   fi
 
-  if check_cmd cppcheck
-  then
-    echo "Running cppcheck"
-    [ -z "${CPPCHECKOPTS+z}" ] && CPPCHECKOPTS="-q --force --enable=warning"
-    find ./src/ -name \*.c | while read -r FILE
-    do
-      [ "$FILE" = "./src/mympd_api/scripts_lualibs.c" ] && continue
-      [ "$FILE" = "./src/web_server/embedded_files.c" ] && continue
-      #shellcheck disable=SC2086
-      if ! cppcheck $CPPCHECKOPTS --error-exitcode=1 "$FILE"
-      then
-        return 1
-      fi
-    done
-    find ./src/ -name \*.h | while read -r FILE
-    do
-      #shellcheck disable=SC2086
-      if ! cppcheck $CPPCHECKOPTS --error-exitcode=1 "$FILE"
-      then
-        return 1
-      fi
-    done
-  else
-    echo_warn "cppcheck not found"
-    return 1
-  fi
-
-  if check_cmd flawfinder
-  then
-    echo "Running flawfinder"
-    [ -z "${FLAWFINDEROPTS+z}" ] && FLAWFINDEROPTS="-m3 --quiet --dataonly"
-    #shellcheck disable=SC2086
-    if ! flawfinder $FLAWFINDEROPTS --error-level=3 src
-    then
-      return 1
-    fi
-    #shellcheck disable=SC2086
-    if ! flawfinder $FLAWFINDEROPTS --error-level=3 cli_tools
-    then
-      return 1
-    fi
-  else
-    echo_warn "flawfinder not found"
-    return 1
-  fi
-
   if [ ! -f src/compile_commands.json ]
   then
     echo "src/compile_commands.json not found"
@@ -563,8 +599,8 @@ check() {
     rm -f clang-tidy.out
     cd src || exit 1
     find ./ -name '*.c' -exec clang-tidy \
-      --checks="$CLANG_TIDY_CHECKS" {} \; >> ../clang-tidy.out 2>/dev/null
-    ERRORS=$(grep -v -E "(/usr/include/|memset|memcpy|\^)" ../clang-tidy.out)
+      --config-file="$STARTPATH/.clang-tidy" {} \; >> ../clang-tidy.out 2>/dev/null
+    ERRORS=$(grep -v -E "(/usr/include/|memset|memcpy|_XOPEN_SOURCE|\^)" ../clang-tidy.out)
     if [ -n "$ERRORS" ]
     then
       echo "$ERRORS"
@@ -589,6 +625,7 @@ prepare() {
     [ "$F" = "$STARTPATH/osc" ] && continue
     [ "$F" = "$STARTPATH/builder" ] && continue
     cp -a "$F" .
+    rm -fr src/.cache
   done
 }
 
@@ -730,7 +767,12 @@ pkgarch() {
 }
 
 pkgosc() {
-  check_cmd osc
+  [ -z "${OSC_BIN+x}" ] && OSC_BIN="$HOME/python-venv/bin/osc"
+  if [ ! -x "$OSC_BIN" ]
+  then
+    echo_error "Command osc not found: $HOME/python-venv/bin/osc"
+    exit 1
+  fi
   cleanup
   cleanuposc
   if [ -z "${OSC_REPO+x}" ]
@@ -745,7 +787,7 @@ pkgosc() {
 
   mkdir osc
   cd osc || exit 1
-  osc checkout "$OSC_REPO"
+  $OSC_BIN checkout "$OSC_REPO"
   rm -f "$OSC_REPO"/*
 
   cd "$STARTPATH" || exit 1
@@ -771,13 +813,12 @@ pkgosc() {
   fi
   cp ../contrib/packaging/rpm/mympd.spec "$OSC_REPO/"
   cp ../contrib/packaging/arch/PKGBUILD "$OSC_REPO/"
-  cp ../contrib/packaging/arch/archlinux.install "$OSC_REPO/"
 
   cd "$OSC_REPO" || exit 1
-  osc addremove
-  osc st
-  osc vc -m "Update"
-  osc commit -m "Update"
+  $OSC_BIN addremove
+  $OSC_BIN st
+  $OSC_BIN vc -m "Update"
+  $OSC_BIN commit -m "Update"
 }
 
 installdeps() {
@@ -785,41 +826,42 @@ installdeps() {
   if [ -f /etc/debian_version ]
   then
     apt-get update
-    if ! apt-get install -y --no-install-recommends liblua5.4-dev
+    if ! apt-get install -y --no-install-recommends liblua5.4-dev lua5.4
     then
       #fallback to lua 5.3 for older debian versions
-      apt-get install -y --no-install-recommends liblua5.3-dev
+      apt-get install -y --no-install-recommends liblua5.3-dev lua5.3
     fi
     apt-get install -y --no-install-recommends \
       gcc cmake perl libssl-dev libid3tag0-dev libflac-dev \
-      build-essential pkg-config libpcre2-dev gzip jq
+      build-essential pkg-config libpcre2-dev gzip jq whiptail
   elif [ -f /etc/arch-release ]
   then
     #arch
-    pacman -Sy gcc base-devel cmake perl openssl libid3tag flac lua pkgconf pcre2 gzip jq
+    pacman -Sy gcc base-devel cmake perl openssl libid3tag flac lua pkgconf pcre2 gzip jq libnewt
   elif [ -f /etc/alpine-release ]
   then
     #alpine
-    apk add cmake perl openssl-dev libid3tag-dev flac-dev lua5.4-dev \
-      alpine-sdk linux-headers pkgconf pcre2-dev gzip jq
+    apk add cmake perl openssl-dev libid3tag-dev flac-dev lua5.4-dev lua5.4 \
+      alpine-sdk linux-headers pkgconf pcre2-dev gzip jq newt
   elif [ -f /etc/SuSE-release ]
   then
     #suse
     zypper install gcc cmake pkgconfig perl openssl-devel libid3tag-devel flac-devel \
-      lua-devel unzip pcre2-devel gzip jq
+      lua-devel unzip pcre2-devel gzip jq whiptail
   elif [ -f /etc/redhat-release ]
   then
     #fedora
     yum install gcc cmake pkgconfig perl openssl-devel libid3tag-devel flac-devel \
-      lua-devel unzip pcre2-devel gzip jq
+      lua-devel unzip pcre2-devel gzip jq whiptail
   else
     echo_warn "Unsupported distribution detected."
     echo "You should manually install:"
-    echo "  - gcc"
+    echo "  - gcc or clang"
     echo "  - cmake"
     echo "  - perl"
     echo "  - gzip"
     echo "  - jq"
+    echo "  - whiptail"
     echo "  - openssl (devel)"
     echo "  - flac (devel)"
     echo "  - libid3tag (devel)"
@@ -831,50 +873,52 @@ installdeps() {
 updatelibmympdclient() {
   check_cmd git meson
 
-  cd dist/libmpdclient || exit 1
+  cd dist/libmympdclient || exit 1
 
   TMPDIR=$(mktemp -d)
   cd "$TMPDIR" || exit 1
-  git clone --depth=1 -b libmympdclient https://github.com/jcorporation/libmpdclient.git
-  cd libmpdclient || exit 1
-  meson . output -Dbuffer_size=8192
+  git clone --depth=1 -b libmympdclient https://github.com/jcorporation/libmympdclient.git
+  cd libmympdclient || exit 1
+  meson setup . output -Dbuffer_size=8192
 
-  cd "$STARTPATH/dist/libmpdclient" || exit 1
+  cd "$STARTPATH/dist/libmympdclient" || exit 1
   install -d src
-  install -d include/mpd/
+  install -d include/mpd
+  install -d LICENSES
 
-  rsync -av --delete "$TMPDIR/libmpdclient/src/" ./src/
-  rsync -av --delete "$TMPDIR/libmpdclient/include/mpd/" ./include/mpd/
+  rsync -av --delete "$TMPDIR/libmympdclient/src/" ./src/
+  rsync -av --delete "$TMPDIR/libmympdclient/include/mpd/" ./include/mpd/
 
-  rsync -av "$TMPDIR/libmpdclient/output/version.h" include/mpd/version.h
-  rsync -av "$TMPDIR/libmpdclient/output/config.h" include/config.h
+  rsync -av "$TMPDIR/libmympdclient/output/include/mpd/version.h" include/mpd/version.h
+  rsync -av "$TMPDIR/libmympdclient/output/config.h" include/config.h
 
-  rsync -av "$TMPDIR/libmpdclient/COPYING" COPYING
-  rsync -av "$TMPDIR/libmpdclient/AUTHORS" AUTHORS
+  rsync -av "$TMPDIR/libmympdclient/LICENSES/" LICENSES/
 
   rm -rf "$TMPDIR"
 }
 
 updatebootstrapnative() {
   check_cmd git npm
-  cd dist/bootstrap-native || exit 1
-
+  #clone repository
   TMPDIR=$(mktemp -d)
   cd "$TMPDIR" || exit 1
-  git clone --depth=1 -b master https://github.com/jcorporation/bootstrap.native.git
+  git clone --depth=1 git@github.com:thednp/bootstrap.native.git
   cd bootstrap.native
-  npm install @rollup/plugin-buble
-  cp "$STARTPATH/dist/bootstrap-native/mympd-config.js" src/
-  cp "$STARTPATH/dist/bootstrap-native/mympd-init.js" src/util/
-  npm run custom INPUTFILE:src/mympd-config.js,OUTPUTFILE:dist/bootstrap-mympd.js,MIN:false,FORMAT:umd
-  npm run custom INPUTFILE:src/mympd-config.js,OUTPUTFILE:dist/bootstrap-mympd.min.js,MIN:true,FORMAT:umd
-
-  cp dist/bootstrap-mympd.js "$STARTPATH/dist/bootstrap-native/bootstrap-native.js"
-  cp dist/bootstrap-mympd.min.js "$STARTPATH/dist/bootstrap-native/bootstrap-native.min.js"
-
+  npm install vite
+  #copy custom config
+  cp "$STARTPATH/dist/bootstrap-native/mympd-config.ts" src/index.ts
+  cp "$STARTPATH/dist/bootstrap-native/mympd-init.ts" src/util/
+  #minified build
+  npm run build-vite
+  grep -v "^//" dist/bootstrap-native.js > "$STARTPATH/dist/bootstrap-native/bootstrap-native.min.js"
+  #normal build
+  sed -i 's/sourcemap: true,/sourcemap: true,\nminify: false/' vite.config.ts
+  npm run build-vite
+  grep -v "^//" dist/bootstrap-native.js > "$STARTPATH/dist/bootstrap-native/bootstrap-native.js"
+  #cleanup
   cd "$STARTPATH" || exit 1
   rm -rf "$TMPDIR"
-
+  #update debug build
   if [ -d debug ]
   then
     cp dist/bootstrap-native/bootstrap-native.js htdocs/js/
@@ -883,16 +927,17 @@ updatebootstrapnative() {
 
 updatebootstrap() {
   check_cmd npm
-  cd dist/bootstrap || exit 1
+  cd "$STARTPATH/dist/bootstrap" || exit 1
   [ -z "${BOOTSTRAP_VERSION+x}" ] && BOOTSTRAP_VERSION=""
   npm install "$BOOTSTRAP_VERSION"
   npm run build
   sed -i '$ d' compiled/custom.css
   rm compiled/custom.css.map
-  if [ -d ../../debug ]
+  if [ -d "$STARTPATH/debug" ]
   then
-    cp -v compiled/custom.css ../../htdocs/css/bootstrap.css
+    cp -v compiled/custom.css "$STARTPATH/htdocs/css/bootstrap.css"
   fi
+  rm -fr  dist/bootstrap/node_modules
 }
 
 #Also deletes stale installations in other locations.
@@ -900,9 +945,9 @@ updatebootstrap() {
 uninstall() {
   #cmake does not provide an uninstall target, instead its manifest is of use at least for
   #the binaries
-  if [ -f release/install_manifest.txt ]
+  if [ -f "$STARTPATH/release/install_manifest.txt" ]
   then
-    xargs rm < release/install_manifest.txt
+    xargs rm -f < "$STARTPATH/release/install_manifest.txt"
   fi
   [ -z "${DESTDIR+x}" ] && DESTDIR=""
   #CMAKE_INSTALL_PREFIX="/usr"
@@ -935,8 +980,13 @@ purge() {
   [ -z "${DESTDIR+x}" ] && DESTDIR=""
   #CMAKE_INSTALL_PREFIX="/usr"
   rm -rf "$DESTDIR/var/lib/mympd"
+  rm -rf "$DESTDIR/var/lib/private/mympd"
   rm -rf "$DESTDIR/var/cache/mympd"
+  rm -rf "$DESTDIR/var/cache/private/mympd"
   rm -f "$DESTDIR/etc/init.d/mympd"
+  rm -rf "$DESTDIR/usr/share/doc/mympd"
+  rm -f "$DESTDIR/usr/share/man/man1/mympd.1.gz"
+  rm -f "$DESTDIR/usr/share/man/man1/mympd-script.1.gz"
   #CMAKE_INSTALL_PREFIX="/opt/mympd/"
   rm -rf "$DESTDIR/var/opt/mympd"
   #remove user
@@ -957,23 +1007,64 @@ purge() {
   #remove group
   if getent group mympd > /dev/null
   then
-    if check_cmd_silent userdel
+    if check_cmd_silent groupdel
     then
-      userdel mympd
-    elif check_cmd_silent deluser
+      groupdel mympd
+    elif check_cmd_silent delgroup
     then
-      deluser mympd
+      #alpine
+      delgroup mympd
     else
-      echo_error "Can not remove user mympd"
+      echo_error "Can not remove group mympd"
       return 1
     fi
   fi
 }
 
+check_phrases() {
+  check_cmd jq
+  echo "Validating translation phrases"
+  for F in "src/i18n/json/"*.json
+  do
+    if ! jq "." "$F" > /dev/null
+    then
+      echo "Invalid json: $F"
+      exit 1
+    fi
+  done
+}
+
+# Create an JavaScript object from compile_time.h.in
+# This is used to define global defaults.
+create_js_defines() {
+  printf "const defaults = " > "$STARTPATH/htdocs/js/defines.js"
+  {
+    I=0
+    printf "{"
+    perl -npe 's/\\\n//g; s/^\s+//g' < "$STARTPATH/src/compile_time.h.in" \
+      | perl -ne 's/MPD_TAG_ARTIST/"Artist"/g; s/""//g; 
+      if (not /\$/ and /^#define ((MYMPD|PARTITION)_\w+)\s+(\S+)/) {
+        print "$1|$3\n";
+      }
+    ' | while IFS="|" read -r KEY VALUE
+      do
+        [ "$I" -gt 0 ] && echo ","
+        if JSON_VALUE=$(printf '{"value": %s}' "$VALUE" | jq -r '.value | fromjson' 2>/dev/null)
+        then
+          printf '"%s":%s' "$KEY" "$JSON_VALUE"
+        else
+          printf '"%s":%s' "$KEY" "$VALUE"
+        fi
+        I=$((I+1))
+      done
+      printf "}"
+  } | jq -r "." >> "$STARTPATH/htdocs/js/defines.js"
+  echo ";" >> "$STARTPATH/htdocs/js/defines.js"
+}
+
 createi18n() {
-  MYMPD_BUILD_DIR="$1"
   check_cmd perl
-  install -d "$MYMPD_BUILD_DIR/htdocs/js"
+  install -d "$MYMPD_BUILDDIR/htdocs/js"
   echo "Creating i18n json"
   if ! perl ./src/i18n/translate.pl
   then
@@ -981,9 +1072,11 @@ createi18n() {
     exit 1
   fi
   #json to js
-  printf "const i18n = " > "$MYMPD_BUILD_DIR/htdocs/js/i18n.js"
-  head -c -1 "src/i18n/json/i18n.json" >> "$MYMPD_BUILD_DIR/htdocs/js/i18n.js"
-  echo ";" >> "$MYMPD_BUILD_DIR/htdocs/js/i18n.js"
+  printf "const i18n = " > "$MYMPD_BUILDDIR/htdocs/js/i18n.js"
+  BYTES=$(wc -c < src/i18n/json/i18n.json)
+  BYTES=$((BYTES-1))
+  head -c "$BYTES" "src/i18n/json/i18n.json" >> "$MYMPD_BUILDDIR/htdocs/js/i18n.js"
+  echo ";" >> "$MYMPD_BUILDDIR/htdocs/js/i18n.js"
   #Update serviceworker
   TO_CACHE=""
   for CODE in $(jq -r "select(.missingPhrases < 100) | keys[]" "$STARTPATH/src/i18n/json/i18n.json" | grep -v "default")
@@ -994,7 +1087,7 @@ createi18n() {
 }
 
 materialicons() {
-  check_cmd jq wget
+  check_cmd jq wget woff2_compress
 
   TMPDIR=$(mktemp -d)
   cd "$TMPDIR" || exit 1
@@ -1005,6 +1098,11 @@ materialicons() {
     echo_error "Error downloading font file"
     exit 1
   fi
+  if ! woff2_compress MaterialIcons-Regular.woff2
+  then
+    echo_error "Compression failed"
+    exit 1
+  fi
   METADATA_URI="https://fonts.google.com/metadata/icons"
   if ! wget -q "$METADATA_URI" -O metadata.json
   then
@@ -1149,6 +1247,8 @@ run_eslint() {
   then
     return 1
   fi
+  # Enforce minification of JavaScript
+  MYMPD_MINIFY_JS=1
   createassets
   rc=0
   echo ""
@@ -1163,7 +1263,7 @@ run_eslint() {
   for F in release/htdocs/sw.min.js release/htdocs/js/mympd.min.js release/htdocs/js/i18n.min.js
   do
     echo "Linting $F"
-    if ! npx eslint --no-eslintrc -c .eslintrc-min.json $F
+    if ! npx eslint $F
     then
       rc=1
     fi
@@ -1193,7 +1293,7 @@ run_stylelint() {
     return 1
   fi
   rc=0
-  for F in mympd.css theme-light.css
+  for F in mympd.css theme-light.css theme-dark.css
   do
     echo "Linting $F"
     if ! npx stylelint "htdocs/css/$F"
@@ -1217,20 +1317,14 @@ run_htmlhint() {
   return 0
 }
 
-luascript_index() {
-  rm -f "docs/scripting/scripts/index.json"
-  exec 3<> "docs/scripting/scripts/index.json"
-  printf "{\"scripts\":[" >&3
-  I=0
-  for F in docs/scripting/scripts/*.lua
-  do
-    [ "$I" -gt 0 ] &&  printf "," >&3
-    SCRIPTNAME=$(basename "$F")
-    printf "\"%s\"" "$SCRIPTNAME" >&3
-    I=$((I+1))
-  done
-  printf "]}\n" >&3
-  exec 3>&-
+run_luacheck() {
+  export MYMPD_ENABLE_MYGPIOD="ON"
+  lualibs
+  if ! luacheck release/contrib/lualibs/
+  then
+    return 1
+  fi
+  return 0
 }
 
 run_doxygen() {
@@ -1244,28 +1338,142 @@ run_doxygen() {
 
 run_jsdoc() {
   if ! check_cmd jsdoc
-  then 
+  then
     return 1
   fi
   echo "Running jsdoc"
   jsdoc htdocs/js/ -c jsdoc.json -d docs/jsdoc/
 }
 
+run_luadoc() {
+  if ! check_cmd luadoc
+  then
+    return 1
+  fi
+  echo "Running luadoc"
+  lualibs
+  luadoc --noindexpage -d docs/luadoc/ release/contrib/lualibs/mympd.lua
+}
+
+create_doc() {
+  DOC_DEST=$1
+  if ! check_cmd jekyll
+  then
+    echo "Jekyll not installed, can not create documentation"
+    return 1
+  fi
+  if ! run_doxygen
+  then
+    echo "Skipped generation of c api documentation"
+  fi
+  if ! run_jsdoc
+  then
+    echo "Skipped generation of js api documentation"
+  fi
+  install -d "$DOC_DEST" || return 1
+  jekyll build -s "$STARTPATH/docs" -d "$DOC_DEST"
+}
+
+translation_import() {
+  #shellcheck disable=SC1091
+  . "$STARTPATH/.secrets"
+  if [ -z "${POEDITOR_TOKEN+x}" ]
+  then
+    echo_error "POEDITOR_TOKEN variable not set."
+    exit 1
+  fi
+  LANG_SHORT=$1
+  LANG_CODE=$2
+  if RESPONSE=$(curl -s -f --show-error -X POST https://api.poeditor.com/v2/projects/export \
+    -d api_token="$POEDITOR_TOKEN" \
+    -d id="550213" \
+    -d language="$LANG_SHORT" \
+    -d type="key_value_json" \
+    -d order="terms")
+  then
+    URL=$(echo "$RESPONSE" | jq -r ".result.url")
+    if [ "$URL" != "null" ]
+    then
+      if curl -s -f --show-error "$URL" -o "src/i18n/json/$LANG_CODE.tmp"
+      then
+        mv "src/i18n/json/$LANG_CODE.tmp" "src/i18n/json/$LANG_CODE.json"
+        return 0
+      fi
+    else
+      echo_error "Download failed: $RESPONSE"
+      return 1
+    fi
+  fi
+  return 1
+}
+
+translation_import_all() {
+  #shellcheck disable=SC1091
+  . "$STARTPATH/.secrets"
+  if [ -z "${POEDITOR_TOKEN+x}" ]
+  then
+    echo_error "POEDITOR_TOKEN variable not set."
+    exit 1
+  fi
+  while IFS=":" read -r LANG_SHORT LANG_CODE LANG_DESC
+  do
+    [ "$LANG_CODE" = "en-US" ] && continue
+    echo "$LANG_SHORT: $LANG_DESC"
+    translation_import "$LANG_SHORT" "$LANG_CODE"
+  done < src/i18n/i18n.txt
+}
+
+terms_export() {
+  #shellcheck disable=SC1091
+  . "$STARTPATH/.secrets"
+  if [ -z "${POEDITOR_TOKEN+x}" ]
+  then
+    echo_error "POEDITOR_TOKEN variable not set."
+    exit 1
+  fi
+  curl -s -f --show-error -X POST https://api.poeditor.com/v2/projects/upload \
+    -F api_token="$POEDITOR_TOKEN" \
+    -F id="550213" \
+    -F updating="terms" \
+    -F sync_terms="1" \
+    -F file=@"src/i18n/json/phrases.json"
+  echo ""
+}
+
+translation_export() {
+  #shellcheck disable=SC1091
+  . "$STARTPATH/.secrets"
+  if [ -z "${POEDITOR_TOKEN+x}" ]
+  then
+    echo_error "POEDITOR_TOKEN variable not set."
+    exit 1
+  fi
+  LANG_SHORT=$1
+  UPLOAD_FILE=$2
+  curl -s -f --show-error -X POST https://api.poeditor.com/v2/projects/upload \
+    -F api_token="$POEDITOR_TOKEN" \
+    -F id="550213" \
+    -F updating="translations" \
+    -F language="$LANG_SHORT" \
+    -F file=@"$UPLOAD_FILE"
+  echo ""
+}
+
 case "$ACTION" in
-  release)
-    buildrelease
+  release|MinSizeRel)
+    buildrelease "Release"
+  ;;
+  RelWithDebInfo)
+    buildrelease "RelWithDebInfo"
   ;;
   install)
     installrelease
   ;;
   releaseinstall)
-    buildrelease
+    buildrelease "Release"
     installrelease
   ;;
-  debug)
-    builddebug
-  ;;
-  memcheck)
+  debug|asan|tsan|ubsan)
     builddebug
   ;;
   test)
@@ -1341,6 +1549,30 @@ case "$ACTION" in
     uninstall
     purge
   ;;
+  terms_export)
+    terms_export
+    ;;
+  trans_export)
+    if [ -z "${2+x}" ] || [ -z "${3+x}" ]
+    then
+      echo "Usage: $0 $1 <code short> <file>"
+      echo "  e.g. $0 $1 de src/i18n/json/de-DE.json"
+      exit 1
+    fi
+    translation_export  "$2" "$3"
+    ;;
+  trans_import)
+    if [ -z "${2+x}" ] || [ -z "${3+x}" ]
+    then
+      echo "Usage: $0 $1 <code short> <lang code full>"
+      echo "  e.g. $0 $1 de de-DE"
+      exit 1
+    fi
+    translation_import "$2" "$3"
+  ;;
+  trans_import_all)
+    translation_import_all
+  ;;
   translate)
     src/i18n/translate.pl verbose
   ;;
@@ -1386,6 +1618,10 @@ case "$ACTION" in
     then
       exit 1
     fi
+    if ! run_luacheck
+    then
+      exit 1
+    fi
   ;;
   eslint)
     run_eslint
@@ -1396,6 +1632,9 @@ case "$ACTION" in
   htmlhint)
     run_htmlhint
   ;;
+  luacheck)
+    run_luacheck
+  ;;
   luascript_index)
     luascript_index
   ;;
@@ -1410,23 +1649,41 @@ case "$ACTION" in
       echo "Could not create frontend api documentation"
       exit 1
     fi
+    if ! run_luadoc
+    then
+      echo "Could not create lua api documentation"
+      exit 1
+    fi
     cp -v htdocs/js/apidoc.js docs/assets/apidoc.js
   ;;
+  doc)
+    if [ -z "${2+x}" ]
+    then
+      echo "Usage: $0 $1 <destination folder>"
+      exit 1
+    fi
+    create_doc "$2"
+    ;;
+  cloc)
+    cloc --exclude-dir=dist .
+  ;;
   *)
     echo "Usage: $0 <option>"
     echo "Version: ${VERSION}"
     echo ""
     echo "Build options:"
-    echo "  release:          build release files in directory release"
+    echo "  release:          build release files in directory release (stripped)"
+    echo "  RelWithDebInfo:   build release files in directory release (with debug info)"
     echo "  install:          installs release files from directory release"
     echo "                    following environment variables are respected"
     echo "                      - DESTDIR=\"\""
     echo "  releaseinstall:   calls release and install afterwards"
     echo "  debug:            builds debug files in directory debug,"
     echo "                    serves assets from htdocs"
-    echo "  memcheck:         builds debug files in directory debug"
-    echo "                    linked with libasan3 and serves assets from htdocs"
-    echo "  test:             builds and runs the unit tests in test/build"
+    echo "  asan|tsan|ubsan:  builds debug files in directory debug"
+    echo "                    linked with the sanitizer and serves assets from htdocs"
+    echo "  test:             builds and runs the unit tests in directory debug"
+    echo "                    linked with libasan3"
     echo "  installdeps:      installs build and runtime dependencies"
     echo "  createassets:     creates the minfied and compressed dist files"
     echo "                    following environment variables are respected"
@@ -1434,17 +1691,18 @@ case "$ACTION" in
     echo "  copyassets:       copies the assets from dist to the source tree"
     echo "                    for debug builds without embedded assets"
     echo "                    following environment variables are respected"
-    echo "                      - MYMPD_BUILDDIR=\"release\""
+    echo "                      - MYMPD_BUILDDIR=\"debug\""
     echo ""
     echo "Translation options:"
     echo "  translate:        builds the translation file for debug builds"
     echo "  transstatus:      shows the translation status"
+    echo "  trans_import:     Import a translation from poeditor.com"
+    echo "  trans_import_all: Import all translations from poeditor.com"
+    echo "  trans_export:     Exports a translation to poeditor.com"
+    echo "  terms_export:     Exports the terms to poeditor.com"
     echo ""
     echo "Check options:"
-    echo "  check:            runs cppcheck, flawfinder and clang-tidy on source files"
-    echo "                    following environment variables are respected"
-    echo "                      - CPPCHECKOPTS=\"-q --force --enable=warning\""
-    echo "                      - FLAWFINDEROPTS=\"-m3 --quiet --dataonly\""
+    echo "  check:            runs clang-tidy on source files"
     echo "  check_file:       same as check, but for one file, second arg must be the file"
     echo "  check_docs        checks the documentation for missing API methods"
     echo "  check_includes:   checks for valid include paths"
@@ -1452,6 +1710,7 @@ case "$ACTION" in
     echo "  eslint:           combines javascript files and runs eslint"
     echo "  stylelint:        runs stylelint (lints css files)"
     echo "  htmlhint:         runs htmlhint (lints html files)"
+    echo "  luacheck:         runs luacheck (lints lua libraries)"
     echo ""
     echo "Cleanup options:"
     echo "  cleanup:          cleanup source tree"
@@ -1473,7 +1732,7 @@ case "$ACTION" in
     echo "                    following environment variables are respected"
     echo "                      - SIGN=\"FALSE\""
     echo "                      - GPGKEYID=\"\""
-    echo "  pkgdocker:        creates the docker image (debian based)"
+    echo "  pkgdocker:        creates the docker image (Alpine Linux based)"
     echo "                    following environment variables are respected"
     echo "                      - DOCKERFILE=\"Dockerfile.alpine\""
     echo "  pkgbuildx:        creates a multiarch docker image with buildx"
@@ -1507,6 +1766,8 @@ case "$ACTION" in
     echo "  addmympduser:     adds mympd group and user"
     echo "  luascript_index:  creates the json index of lua scripts"
     echo "  api_doc:          generates the api documentation"
+    echo "  doc:              generates the html documentation"
+    echo "  cloc:             runs cloc (count lines of code)"
     echo ""
     echo "Source update options:"
     echo "  bootstrap:        updates bootstrap"
