@@ -1,6 +1,6 @@
 "use strict";
 // SPDX-License-Identifier: GPL-3.0-or-later
-// myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+// myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
 // https://github.com/jcorporation/mympd
 
 /** @module elements_js */
@@ -19,7 +19,7 @@ function elCreateTextTnNr(tagName, attributes, text, smartCount) {
 }
 
 /**
- * Creates and translates a html element
+ * Creates a html element and translates the text
  * @param {string} tagName name of the tag to create
  * @param {object} attributes tag attributes
  * @param {string} text text phrase to translate
@@ -45,7 +45,7 @@ function elCreateTextTn(tagName, attributes, text, data) {
  * Creates a html element with text content
  * @param {string} tagName name of the tag
  * @param {object} attributes tag attributes
- * @param {string} text text phrase to translate
+ * @param {string} text text for the elements, respects \n for newlines
  * @returns {HTMLElement} created dom node
  */
 function elCreateText(tagName, attributes, text) {
@@ -53,7 +53,18 @@ function elCreateText(tagName, attributes, text) {
         attributes["title"] = tn(attributes["data-title-phrase"]);
     }
     const tag = elCreateEmpty(tagName, attributes);
-    tag.textContent = text;
+    if (text.length > 0) {
+        const lines = text.split(/\n/);
+        for (let i = 0, j = lines.length; i < j; i++) {
+            if (i > 0) {
+                tag.appendChild(document.createElement('br'));
+            }
+            tag.appendChild(document.createTextNode(lines[i]));
+        }
+    }
+    else {
+        tag.textContent = text;
+    }
     return tag;
 }
 
@@ -114,48 +125,63 @@ function elCreateEmpty(tagName, attributes) {
  * Clears the element with given id and appends the new child
  * @param {string} id id of the parent element
  * @param {Element | Node} child element to add
+ * @returns {void}
  */
 function elReplaceChildId(id, child) {
-    elReplaceChild(document.getElementById(id), child);
+    elReplaceChild(elGetById(id), child);
 }
 
 /**
  * Clears the given element and appends the new child
  * @param {Element} el id of the parent element
  * @param {Element | Node} child element to add
+ * @returns {void}
  */
 function elReplaceChild(el, child) {
     elClear(el);
     el.appendChild(child);
 }
 
+/**
+ * Shortcut for elGetById
+ * @param {string} id element id
+ * @returns {HTMLElement} found element
+ */
+function elGetById(id) {
+    return document.getElementById(id);
+}
+
 /**
  * Hides the element with the given id
  * @param {string} id element id
+ * @returns {void}
  */
 function elHideId(id) {
-    document.getElementById(id).classList.add('d-none');
+    elGetById(id).classList.add('d-none');
 }
 
 /**
  * Shows the element with the given id
  * @param {string} id element id
+ * @returns {void}
  */
 function elShowId(id) {
-    document.getElementById(id).classList.remove('d-none');
+    elGetById(id).classList.remove('d-none');
 }
 
 /**
  * Clears the element with the given id
  * @param {string} id element id to clear
+ * @returns {void}
  */
 function elClearId(id) {
-    document.getElementById(id).textContent = '';
+    elGetById(id).textContent = '';
 }
 
 /**
  * Hides the element
  * @param {Element | EventTarget} el element to hide
+ * @returns {void}
  */
 function elHide(el) {
     el.classList.add('d-none');
@@ -164,6 +190,7 @@ function elHide(el) {
 /**
  * Shows the element
  * @param {Element} el element to show
+ * @returns {void}
  */
 function elShow(el) {
     el.classList.remove('d-none');
@@ -171,7 +198,8 @@ function elShow(el) {
 
 /**
  * Clears the element
- * @param {Element} el element to clear
+ * @param {Element | ChildNode} el element to clear
+ * @returns {void}
  */
 function elClear(el) {
     el.textContent = '';
@@ -180,33 +208,35 @@ function elClear(el) {
 /**
  * Disables the element with the given id
  * @param {string} id element id
+ * @returns {void}
  */
 function elDisableId(id) {
-    document.getElementById(id).setAttribute('disabled', 'disabled');
+    elDisable(elGetById(id));
 }
 
 /**
  * Disables the element
  * @param {Node} el element to disable
+ * @returns {void}
  */
 function elDisable(el) {
     el.setAttribute('disabled', 'disabled');
-    //manually disabled, remove disabled class
-    el.classList.remove('disabled');
     el.classList.replace('clickable', 'not-clickable');
 }
 
 /**
  * Enables the element with the given id
  * @param {string} id element id
+ * @returns {void}
  */
 function elEnableId(id) {
-    document.getElementById(id).removeAttribute('disabled');
+    elEnable(elGetById(id));
 }
 
 /**
  * Enables the element
  * @param {Element | Node} el element to enable
+ * @returns {void}
  */
 function elEnable(el) {
     el.removeAttribute('disabled');
@@ -223,16 +253,18 @@ function elReflow(el) {
 }
 
 /**
- * Sets the focus on the element with given id
+ * Sets the focus on the element with given id for desktop view.
  * @param {string} id element id
+ * @returns {void}
  */
  function setFocusId(id) {
-    setFocus(document.getElementById(id));
+    setFocus(elGetById(id));
 }
 
 /**
- * Set the focus on the given element.
+ * Set the focus on the given element for desktop view.
  * @param {HTMLElement} el element to focus
+ * @returns {void}
  */
 function setFocus(el) {
     if (userAgentData.isMobile === false) {
@@ -244,37 +276,61 @@ function setFocus(el) {
  * Sets an attribute on the element given by id.
  * @param {string} id element id
  * @param {string} attribute attribute name
- * @param {*} value could be any type
+ * @param {object} value could be any type
+ * @returns {void}
  */
 function setDataId(id, attribute, value) {
-    document.getElementById(id)['myMPD-' + attribute] = value;
+    elGetById(id)['myMPD-' + attribute] = value;
 }
 
 /**
  * Sets an attribute on the given element.
- * @param {Element | Node} el element
+ * @param {Element | Node | EventTarget} el element
  * @param {string} attribute attribute name
- * @param {*} value could be any type
+ * @param {object} value could be any type
+ * @returns {void}
  */
 function setData(el, attribute, value) {
     el['myMPD-' + attribute] = value;
 }
 
+/**
+ * Removes an attribute on the element given by id.
+ * @param {string} id element id
+ * @param {string} attribute attribute name
+ * @returns {void}
+ */
+//eslint-disable-next-line no-unused-vars
+function rmDataId(id, attribute) {
+    elGetById(id)['myMPD-' + attribute] = undefined;
+}
+
+/**
+ * Removes an attribute on the given element.
+ * @param {Element | Node} el element
+ * @param {string} attribute attribute name
+ * @returns {void}
+ */
+//eslint-disable-next-line no-unused-vars
+function rmData(el, attribute) {
+    el['myMPD-' + attribute] = undefined;
+}
+
 /**
  * Gets the attributes value from the element given by id.
  * @param {string} id element id
  * @param {string} attribute attribute name
- * @returns {*} attribute value
+ * @returns {object} attribute value
  */
 function getDataId(id, attribute) {
-    return getData(document.getElementById(id), attribute);
+    return getData(elGetById(id), attribute);
 }
 
 /**
  * Gets the attributes value from the element
  * @param {Element | EventTarget} el element
  * @param {string} attribute attribute name
- * @returns {*} attribute value
+ * @returns {object} attribute value or undefined
  */
 function getData(el, attribute) {
     let value = el['myMPD-' + attribute];
@@ -296,11 +352,12 @@ function getData(el, attribute) {
  * @returns {string} selected option value
  */
 function getSelectValueId(id) {
-    return getSelectValue(document.getElementById(id));
+    return getSelectValue(elGetById(id));
 }
 
 /**
  * Gets the value of the selected option of a select element
+ * or undefined if no option is selected
  * @param {Element | EventTarget} el element
  * @returns {string} selected option value
  */
@@ -315,17 +372,17 @@ function getSelectValue(el) {
  * Gets the attribute value of the selected option of a select element
  * @param {string} id element id
  * @param {string} attribute attribute name
- * @returns {*} selected option data value
+ * @returns {object} selected option data value
  */
 function getSelectedOptionDataId(id, attribute) {
-    return getSelectedOptionData(document.getElementById(id), attribute)
+    return getSelectedOptionData(elGetById(id), attribute);
 }
 
 /**
  * Gets the attribute value of the selected option of a select element
  * @param {Element} el element
  * @param {string} attribute attribute name
- * @returns {*} selected option data value
+ * @returns {object} selected option data value
  */
 function getSelectedOptionData(el, attribute) {
     if (el && el.selectedIndex >= 0) {
@@ -340,7 +397,7 @@ function getSelectedOptionData(el, attribute) {
  * @returns {string} radio box value
  */
 function getRadioBoxValueId(id) {
-    return getRadioBoxValue(document.getElementById(id));
+    return getRadioBoxValue(elGetById(id));
 }
 
 /**
@@ -355,6 +412,7 @@ function getRadioBoxValue(el) {
             return button.value;
         }
     }
+    return null;
 }
 
 /**
@@ -386,38 +444,29 @@ function getYpos(el) {
 }
 
 /**
- * Returns the nearest parent of type nodeName
- * @param {HTMLElement | EventTarget} el start element for search
- * @param {string} nodeName nodeName to search
- * @returns {HTMLElement} the nearest parent node with the given nodeName
+ * Gets the index of the element in the parent html collection
+ * @param {HTMLElement} el element to get the index
+ * @returns {number} the index
  */
- function getParent(el, nodeName) {
-    let target = el;
-    let i = 0;
-    while (target.nodeName !== nodeName) {
-        i++;
-        if (i > 10) {
-            return null;
-        }
-        target = target.parentNode;
-    }
-    // @ts-ignore
-    return target;
+function elGetIndex(el) {
+    return [...el.parentNode.children].indexOf(el);
 }
 
 /**
  * Adds a waiting animation to a button
  * @param {string} id id of the button
  * @param {boolean} waiting true = add animation, false = remove animation
+ * @returns {void}
  */
  function btnWaitingId(id, waiting) {
-    btnWaiting(document.getElementById(id), waiting);
+    btnWaiting(elGetById(id), waiting);
 }
 
 /**
  * Adds a waiting animation to a button
- * @param {HTMLElement} btn id of the button
+ * @param {Node} btn id of the button
  * @param {boolean} waiting true = add animation, false = remove animation
+ * @returns {void}
  */
 function btnWaiting(btn, waiting) {
     if (waiting === true) {
@@ -448,7 +497,7 @@ function btnWaiting(btn, waiting) {
  * @returns {HTMLElement} selected button
  */
 function toggleBtnGroupValueId(id, value) {
-    return toggleBtnGroupValue(document.getElementById(id), value)
+    return toggleBtnGroupValue(elGetById(id), value);
 }
 
 /**
@@ -481,14 +530,15 @@ function toggleBtnGroupValue(btngrp, value) {
  * @param {Element} btngrp button group to toggle
  * @param {string} collapseId id of element to collapse
  * @param {string | number} value value to select
+ * @returns {void}
  */
 function toggleBtnGroupValueCollapse(btngrp, collapseId, value) {
     const activeBtn = toggleBtnGroupValue(btngrp, value);
     if (activeBtn.getAttribute('data-collapse') === 'show') {
-        document.getElementById(collapseId).classList.add('show');
+        elGetById(collapseId).classList.add('show');
     }
     else {
-        document.getElementById(collapseId).classList.remove('show');
+        elGetById(collapseId).classList.remove('show');
     }
 }
 
@@ -499,7 +549,7 @@ function toggleBtnGroupValueCollapse(btngrp, collapseId, value) {
  */
 //eslint-disable-next-line no-unused-vars
 function toggleBtnGroupId(id) {
-    return toggleBtnGroup(document.getElementById(id));
+    return toggleBtnGroup(elGetById(id));
 }
 
 /**
@@ -524,12 +574,13 @@ function toggleBtnGroup(btn) {
  * Toggles a button group by triggering element and toggle a collapse
  * @param {HTMLElement} el triggering button 
  * @param {string} collapseId id of element to collapse
+ * @returns {void}
  */
 //eslint-disable-next-line no-unused-vars
 function toggleBtnGroupCollapse(el, collapseId) {
     const activeBtn = toggleBtnGroup(el);
     if (activeBtn.getAttribute('data-collapse') === 'show') {
-        if (document.getElementById(collapseId).classList.contains('show') === false) {
+        if (elGetById(collapseId).classList.contains('show') === false) {
             uiElements[collapseId].show();
         }
     }
@@ -541,7 +592,7 @@ function toggleBtnGroupCollapse(el, collapseId) {
 /**
  * Gets the value from the active button in a button group
  * @param {string} id id of the button group
- * @returns {*} value the value of the active button
+ * @returns {object} value the value of the active button
  */
 function getBtnGroupValueId(id) {
     let activeBtn = document.querySelector('#' + id + ' > .active');
@@ -555,17 +606,19 @@ function getBtnGroupValueId(id) {
 /**
  * Toggles the active state of a button
  * @param {string} id id of button to toggle
- * @param {*} state true = active, false = inactive 
+ * @param {boolean | number} state true, 1 = active; false, 0 = inactive
+ * @returns {void}
  */
 //eslint-disable-next-line no-unused-vars
 function toggleBtnId(id, state) {
-    toggleBtn(document.getElementById(id), state);
+    toggleBtn(elGetById(id), state);
 }
 
 /**
  * Toggles the active state of a button
  * @param {HTMLElement | EventTarget} btn button to toggle
- * @param {boolean | number} state true = active, false = inactive 
+ * @param {boolean | number} state true, 1 = active; false, 0 = inactive
+ * @returns {void}
  */
 function toggleBtn(btn, state) {
     if (state === undefined) {
@@ -573,9 +626,7 @@ function toggleBtn(btn, state) {
         state = btn.classList.contains('active') ? false : true;
     }
 
-    if (state === true ||
-        state === 1)
-    {
+    if (state === true) {
         btn.classList.add('active');
     }
     else {
@@ -583,19 +634,64 @@ function toggleBtn(btn, state) {
     }
 }
 
+/**
+ * Mirrors the button horizontal
+ * @param {string} id button id to mirror
+ * @param {boolean} mirror true = mirror, false = not
+ * @returns {void}
+ */
+function mirrorBtnId(id, mirror) {
+    mirrorBtn(elGetById(id), mirror);
+}
+
+/**
+ * Mirrors the button horizontal
+ * @param {HTMLElement | EventTarget} btn button to mirror
+ * @param {boolean} mirror true = mirror, false = not
+ * @returns {void}
+ */
+function mirrorBtn(btn, mirror) {
+    if (mirror === true) {
+        btn.classList.add('mirror');
+    }
+    else {
+        btn.classList.remove('mirror');
+    }
+}
+
+/**
+ * Gets the enabled state of a check button
+ * @param {string} id check button id
+ * @returns {boolean} enabled = true, disabled = false
+ */
+//eslint-disable-next-line no-unused-vars
+function getBtnChkValueId(id) {
+    return getBtnChkValue(elGetById(id));
+}
+
+/**
+ * Gets the enabled state of a check button
+ * @param {HTMLElement | EventTarget} btn check button id
+ * @returns {boolean} enabled = true, disabled = false
+ */
+function getBtnChkValue(btn) {
+    return btn.classList.contains('active');
+}
+
 /**
  * Toggles a check button
  * @param {string} id id of the button to toggle
- * @param {boolean} state true = active, false = inactive 
+ * @param {boolean} state true, 1 = active; false, 0 = inactive
+ * @returns {void}
  */
 function toggleBtnChkId(id, state) {
-    toggleBtnChk(document.getElementById(id), state);
+    toggleBtnChk(elGetById(id), state);
 }
 
 /**
  * Toggles a check button
  * @param {HTMLElement | EventTarget} btn the button to toggle
- * @param {boolean | number} state true = active, false = inactive 
+ * @param {boolean | number} state true, 1 = active; false, 0 = inactive
  * @returns {boolean} true if button is checked, else false
  */
 function toggleBtnChk(btn, state) {
@@ -604,9 +700,7 @@ function toggleBtnChk(btn, state) {
         state = btn.classList.contains('active') ? false : true;
     }
 
-    if (state === true ||
-        state === 1)
-    {
+    if (state === true) {
         btn.classList.add('active');
         btn.textContent = 'check';
         return true;
@@ -622,51 +716,62 @@ function toggleBtnChk(btn, state) {
  * Toggles a check button and an assigned collapse
  * @param {string} id the id of the triggering button
  * @param {string} collapseId id of element to collapse
- * @param {boolean | number} state true = active, false = inactive 
+ * @param {boolean | number} state true = active, false = inactive
+ * @returns {void}
  */
 function toggleBtnChkCollapseId(id, collapseId, state) {
-    toggleBtnChkCollapse(document.getElementById(id), collapseId, state);
+    toggleBtnChkCollapse(elGetById(id), collapseId, state);
 }
 
 /**
  * Toggles a check button and an assigned collapse
  * @param {HTMLElement} btn triggering button
  * @param {string} collapseId id of element to collapse
- * @param {boolean | number} state true = active, false = inactive 
+ * @param {boolean | number} state true = active, false = inactive
+ * @returns {void}
  */
 function toggleBtnChkCollapse(btn, collapseId, state) {
     const checked = toggleBtnChk(btn, state);
     if (checked === true) {
-        document.getElementById(collapseId).classList.add('show');
+        elGetById(collapseId).classList.add('show');
     }
     else {
-        document.getElementById(collapseId).classList.remove('show');
+        elGetById(collapseId).classList.remove('show');
     }
 }
 
 /**
  * Gets the y-scrolling position
+ * @param {HTMLElement | Element} [el] element
  * @returns {number} the vertical scrolling position
  */
- function getScrollPosY() {
+function getScrollPosY(el) {
+    // element in scrolling modal
+    if (el) {
+        const modal = el.closest('.modal');
+        if (modal) {
+            let scrollPos = window.scrollY;
+            scrollPos += modal.scrollTop;
+            return scrollPos;
+        }
+    }
     if (userAgentData.isMobile === true) {
+        // scrolling body
         return document.body.scrollTop ? document.body.scrollTop : document.documentElement.scrollTop;
     }
-    else {
-        const container = document.getElementById(app.id + 'List');
-        if (container) {
-            return container.parentNode.scrollTop;
-        }
-        else {
-            return 0;
-        }
+    // scrolling container
+    const container = elGetById(app.id + 'List');
+    if (container) {
+        return container.parentNode.scrollTop;
     }
+    return 0;
 }
 
 /**
  * Scrolls the container or the window to the y-position
  * @param {Element | ParentNode} container or null
  * @param {number} pos position to scroll to
+ * @returns {void}
  */
 function scrollToPosY(container, pos) {
     if (userAgentData.isMobile === true ||
