@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -19,22 +19,29 @@
 #include <netinet/in.h>
 #include <openssl/bn.h>
 #include <openssl/conf.h>
+#include <openssl/ec.h>
 #include <openssl/err.h>
 #include <openssl/evp.h>
+#include <openssl/opensslv.h>
 #include <openssl/pem.h>
 #include <openssl/rand.h>
+#include <openssl/rsa.h>
 #include <openssl/x509v3.h>
 #include <string.h>
+#include <sys/socket.h>
 #include <unistd.h>
 
 //private definitions
+static sds print_asn1_time(sds buffer, const ASN1_TIME *time);
+static sds print_x509_subject(sds buffer, X509 *cert);
 static bool certificates_create(sds dir, sds custom_san);
+static void push_san(struct t_list *san_list, const char *san);
 static sds get_san(sds buffer);
 static bool generate_set_random_serial(X509 *cert);
 static X509_REQ *generate_request(EVP_PKEY *pkey);
 static void add_extension(X509V3_CTX *ctx, X509 *cert, int nid, const char *value);
 static X509 *sign_certificate_request(EVP_PKEY *ca_key, X509 *ca_cert, X509_REQ *req, sds san);
-static EVP_PKEY *generate_keypair(int rsa_key_bits);
+static EVP_PKEY *generate_keypair(int key_type, unsigned key_bits);
 static X509 *generate_selfsigned_cert(EVP_PKEY *pkey);
 static bool write_to_disk(sds key_file, EVP_PKEY *pkey, sds cert_file, X509 *cert);
 static bool load_certificate(sds key_file, EVP_PKEY **key, sds cert_file, X509 **cert);
@@ -62,14 +69,14 @@ enum expire_check_rcs {
  * @return true on success else false
  */
 bool certificates_check(sds workdir, sds ssl_san) {
-    sds testdirname = sdscatfmt(sdsempty(), "%S/ssl", workdir);
+    sds testdirname = sdscatfmt(sdsempty(), "%S/%s", workdir, DIR_WORK_SSL);
     int testdir_rc = testdir("SSL cert dir", testdirname, true, false);
     if (testdir_rc == DIR_EXISTS ||
         testdir_rc == DIR_CREATED)
     {
         if (certificates_create(testdirname, ssl_san) == false) {
             //error creating certificates
-            MYMPD_LOG_ERROR("Certificate creation failed");
+            MYMPD_LOG_ERROR(NULL, "Certificate creation failed");
             FREE_SDS(testdirname);
             return false;
         }
@@ -82,10 +89,81 @@ bool certificates_check(sds workdir, sds ssl_san) {
     return true;
 }
 
+/**
+ * Returns the certificate details
+ * @param cert_content the certificate as pem encoded sds
+ * @return certificate details as newly allocated sds
+ */
+sds certificate_get_detail(sds cert_content) {
+    BIO *bio = BIO_new(BIO_s_mem());
+    BIO_puts(bio, cert_content);
+    X509 *cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
+    sds details = sdsempty();
+    if (cert == NULL) {
+        MYMPD_LOG_ERROR(NULL, "Failure parsing the x509 certificate");
+    }
+    else {
+        details = print_x509_subject(details, cert);
+        const ASN1_TIME *not_after = X509_get0_notAfter(cert);
+        const ASN1_TIME *not_before = X509_get0_notBefore(cert);
+        details = sdscat(details, ", valid from ");
+        details = print_asn1_time(details, not_before);
+        details = sdscat(details, " to ");
+        details = print_asn1_time(details, not_after);
+        X509_free(cert);
+    }
+    BIO_free_all(bio);
+    return details;
+}
+
 /**
  * Private functions
  */
 
+/**
+ * Prints an asn1 time struct to buffer
+ * @param buffer already allocated sds string
+ * @param time asn1 time struct to print
+ * @return pointer to buffer
+ */
+static sds print_asn1_time(sds buffer, const ASN1_TIME *time) {
+    BIO *out = BIO_new(BIO_s_mem());
+    if (ASN1_TIME_print(out, time) == 1) {
+        unsigned char *string;
+        long n = BIO_get_mem_data(out, &string);
+        if (n > 0) {
+            buffer = sdscatlen(buffer, string, (size_t)n);
+        }
+    }
+    else {
+        MYMPD_LOG_ERROR(NULL, "Failure reading time");
+    }
+    BIO_free(out);
+    return buffer;
+}
+
+/**
+ * Prints x509 subject to buffer
+ * @param buffer already allocated sds string
+ * @param cert x509 certificate
+ * @return pointer to buffer
+ */
+static sds print_x509_subject(sds buffer, X509 *cert) {
+    BIO *out = BIO_new(BIO_s_mem());
+    if (X509_NAME_print_ex(out, X509_get_subject_name(cert), 0, XN_FLAG_RFC2253) > -1) {
+        unsigned char *subject;
+        long n = BIO_get_mem_data(out, &subject);
+        if (n > 0) {
+            buffer = sdscatlen(buffer, subject, (size_t)n);
+        }
+    }
+    else {
+        MYMPD_LOG_ERROR(NULL, "Failure reading subject name");
+    }
+    BIO_free(out);
+    return buffer;
+}
+
 /**
  * Creates the ca and cert and renews before expired
  * @param dir key and certificate directory
@@ -106,7 +184,7 @@ static bool certificates_create(sds dir, sds custom_san) {
         rc_ca = create_ca_certificate(cakey_file, &ca_key, cacert_file, &ca_cert);
     }
     else {
-        MYMPD_LOG_NOTICE("CA certificate and private key found");
+        MYMPD_LOG_NOTICE(NULL, "CA certificate and private key found");
         int rc_expires = check_expiration(ca_cert, cacert_file, CA_LIFETIME_MIN, CA_LIFETIME);
         if (rc_expires == CERT_EXPIRE_OK) {
             rc_ca = true;
@@ -140,7 +218,7 @@ static bool certificates_create(sds dir, sds custom_san) {
             custom_san, &ca_key, &ca_cert);
     }
     else {
-        MYMPD_LOG_NOTICE("Server certificate and private key found");
+        MYMPD_LOG_NOTICE(NULL, "Server certificate and private key found");
         int rc_expires = check_expiration(server_cert, servercert_file, CERT_LIFETIME_MIN, CERT_LIFETIME);
         if (rc_expires == CERT_EXPIRE_OK) {
             rc_cert = true;
@@ -200,21 +278,21 @@ static bool certificates_cleanup(sds dir, const char *name) {
  *         CERT_EXPIRE_RENEW cert must be renewed
  */
 static int check_expiration(X509 *cert, sds cert_file, int min_days, int max_days) {
-    ASN1_TIME *not_after = X509_get_notAfter(cert);
+    const ASN1_TIME *not_after = X509_get0_notAfter(cert);
     int pday = 0;
     int psec = 0;
     int rc = ASN1_TIME_diff(&pday, &psec, NULL, not_after);
     if (rc == 1) {
-        MYMPD_LOG_DEBUG("Certificate %s expires in %d days", cert_file, pday);
+        MYMPD_LOG_DEBUG(NULL, "Certificate %s expires in %d days", cert_file, pday);
         if (pday > max_days ||
             pday < min_days)
         {
-            MYMPD_LOG_WARN("Certificate %s must be renewed, expires in %d days", cert_file, pday);
+            MYMPD_LOG_WARN(NULL, "Certificate %s must be renewed, expires in %d days", cert_file, pday);
             return CERT_EXPIRE_RENEW;
         }
     }
     else {
-        MYMPD_LOG_ERROR("Can not parse date from certificate file: %s", cert_file);
+        MYMPD_LOG_ERROR(NULL, "Can not parse date from certificate file: %s", cert_file);
         return CERT_EXPIRE_ERROR;
     }
     return CERT_EXPIRE_OK;
@@ -229,8 +307,8 @@ static int check_expiration(X509 *cert, sds cert_file, int min_days, int max_day
  * @return true on success else false
  */
 static bool create_ca_certificate(sds cakey_file, EVP_PKEY **ca_key, sds cacert_file, X509 **ca_cert) {
-    MYMPD_LOG_NOTICE("Creating self signed ca certificate");
-    *ca_key = generate_keypair(CA_KEY_LENGTH);
+    MYMPD_LOG_NOTICE(NULL, "Creating self signed ca certificate");
+    *ca_key = generate_keypair(CA_KEY_TYPE, CA_KEY_LENGTH);
     if (*ca_key == NULL) {
         return false;
     }
@@ -258,8 +336,8 @@ static bool create_server_certificate(sds serverkey_file, EVP_PKEY **server_key,
         sds servercert_file, X509 **server_cert, sds custom_san, EVP_PKEY **ca_key,
         X509 **ca_cert)
 {
-    MYMPD_LOG_NOTICE("Creating server certificate");
-    *server_key = generate_keypair(CERT_KEY_LENGTH);
+    MYMPD_LOG_NOTICE(NULL, "Creating server certificate");
+    *server_key = generate_keypair(CERT_KEY_TYPE, CERT_KEY_LENGTH);
     if (*server_key == NULL) {
         return false;
     }
@@ -271,10 +349,10 @@ static bool create_server_certificate(sds serverkey_file, EVP_PKEY **server_key,
     sds san = sdsempty();
     san = get_san(san);
     if (sdslen(custom_san) > 0) {
-        MYMPD_LOG_DEBUG("Adding custom san: %s", custom_san);
+        MYMPD_LOG_DEBUG(NULL, "Adding custom san: %s", custom_san);
         san = sdscatfmt(san, ",%S", custom_san);
     }
-    MYMPD_LOG_NOTICE("Set server certificate san to: %s", san);
+    MYMPD_LOG_NOTICE(NULL, "Set server certificate san to: %s", san);
     *server_cert = sign_certificate_request(*ca_key, *ca_cert, server_req, san);
     X509_REQ_free(server_req);
     if (*server_cert == NULL) {
@@ -294,37 +372,49 @@ static bool create_server_certificate(sds serverkey_file, EVP_PKEY **server_key,
  * @param cert pointer to X509 struct to populate
  */
 static bool load_certificate(sds key_file, EVP_PKEY **key, sds cert_file, X509 **cert) {
-	BIO *bio = NULL;
-	*cert = NULL;
-	*key = NULL;
+    BIO *bio = NULL;
+    *cert = NULL;
+    *key = NULL;
 
-	/* Load private key. */
-	bio = BIO_new(BIO_s_file());
-	if (!BIO_read_filename(bio, key_file)) {
-	    BIO_free_all(bio);
-	    return false;
-	}
-	*key = PEM_read_bio_PrivateKey(bio, NULL, NULL, NULL);
-	BIO_free_all(bio);
-	if (*key == NULL) {
-	    return false;
-	}
+    /* Load private key. */
+    bio = BIO_new(BIO_s_file());
+    if (!BIO_read_filename(bio, key_file)) {
+        BIO_free_all(bio);
+        return false;
+    }
+    *key = PEM_read_bio_PrivateKey(bio, NULL, NULL, NULL);
+    BIO_free_all(bio);
+    if (*key == NULL) {
+        return false;
+    }
 
-	/* Load certificate. */
-	bio = BIO_new(BIO_s_file());
-	if (!BIO_read_filename(bio, cert_file)) {
-	    BIO_free_all(bio);
-	    EVP_PKEY_free(*key);
-	    return false;
-	}
-	*cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
-	BIO_free_all(bio);
-	if (!*cert) {
-	    EVP_PKEY_free(*key);
-	    return false;
-	}
+    /* Load certificate. */
+    bio = BIO_new(BIO_s_file());
+    if (!BIO_read_filename(bio, cert_file)) {
+        BIO_free_all(bio);
+        EVP_PKEY_free(*key);
+        return false;
+    }
+    *cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
+    BIO_free_all(bio);
+    if (!*cert) {
+        EVP_PKEY_free(*key);
+        return false;
+    }
 
-	return true;
+    return true;
+}
+
+/**
+ * Adds a uniq string to the san list
+ * @param san_list pointer to the san list
+ * @param san string to add
+ */
+static void push_san(struct t_list *san_list, const char *san) {
+    if (list_get_node(san_list, san) == NULL) {
+        MYMPD_LOG_DEBUG(NULL, "Adding %s to SAN", san);
+        list_push(san_list, san, 0, NULL, NULL);
+    }
 }
 
 /**
@@ -336,33 +426,27 @@ static sds get_san(sds buffer) {
     sds key = sdsempty();
     struct t_list san;
     list_init(&san);
-    MYMPD_LOG_DEBUG("Adding DNS:localhost to SAN");
-    list_push(&san, "DNS:localhost", 0, NULL, NULL);
+    push_san(&san, "DNS:localhost");
     #ifdef MYMPD_ENABLE_IPV6
-        list_push(&san, "DNS:ip6-localhost", 0, NULL, NULL);
-        list_push(&san, "DNS:ip6-loopback", 0, NULL, NULL);
+        push_san(&san, "DNS:ip6-localhost");
+        push_san(&san, "DNS:ip6-loopback");
     #endif
 
     //Retrieve short hostname
     char hostbuffer[256]; /* Flawfinder: ignore */
     int hostname = gethostname(hostbuffer, sizeof(hostbuffer));
     if (hostname == 0) {
-        MYMPD_LOG_DEBUG("Adding DNS:%s to SAN", hostbuffer);
         key = sdscatfmt(key, "DNS:%s", hostbuffer);
-        list_push(&san, key, 0, NULL, NULL);
+        push_san(&san, key);
         //Retrieve fqdn
         struct addrinfo hints = {0};
         hints.ai_family = AF_UNSPEC;
         hints.ai_flags = AI_CANONNAME;
         struct addrinfo *res;
         if (getaddrinfo(hostbuffer, 0, &hints, &res) == 0) {
-            // The hostname was successfully resolved.
-            if (strcmp(hostbuffer, res->ai_canonname) != 0) {
-                MYMPD_LOG_DEBUG("Adding DNS:%s to SAN", res->ai_canonname);
-                sdsclear(key);
-                key = sdscatfmt(key, "DNS:%s", res->ai_canonname);
-                list_push(&san, key, 0, NULL, NULL);
-            }
+            sdsclear(key);
+            key = sdscatfmt(key, "DNS:%s", res->ai_canonname);
+            push_san(&san, key);
             freeaddrinfo(res);
         }
     }
@@ -388,23 +472,22 @@ static sds get_san(sds buffer) {
                     NULL, 0, NI_NUMERICHOST);
 
                 if (s != 0) {
-                    MYMPD_LOG_ERROR("getnameinfo() failed: %s\n", gai_strerror(s));
+                    MYMPD_LOG_ERROR(NULL, "getnameinfo() failed: %s\n", gai_strerror(s));
                     continue;
                 }
                 sdsclear(key);
                 char *crap;
-                // remove zone info from ipv6
+                //remove zone info from ipv6
                 char *ip = strtok_r(host, "%", &crap);
-                MYMPD_LOG_DEBUG("Adding IP:%s to SAN", ip);
                 key = sdscatfmt(key, "IP:%s", ip);
-                list_push(&san, key, 0, NULL, NULL);
+                push_san(&san, key);
             }
         }
         freeifaddrs(ifaddr);
     }
     else {
-        MYMPD_LOG_ERROR("Can not get list of inteface ip addresses");
-        MYMPD_LOG_ERRNO(errno);
+        MYMPD_LOG_ERROR(NULL, "Can not get list of inteface ip addresses");
+        MYMPD_LOG_ERRNO(NULL, errno);
     }
     //create san string
     struct t_list_node *current = san.head;
@@ -452,14 +535,14 @@ static bool generate_set_random_serial(X509 *cert) {
 static X509_REQ *generate_request(EVP_PKEY *pkey) {
     X509_REQ *req = X509_REQ_new();
     if (!req) {
-        MYMPD_LOG_ERROR("Unable to create X509_REQ structure");
+        MYMPD_LOG_ERROR(NULL, "Unable to create X509_REQ structure");
         return NULL;
     }
     X509_REQ_set_pubkey(req, pkey);
 
     /* Set the DN */
     time_t now = time(NULL);
-    sds cn = sdscatfmt(sdsempty(), "myMPD Server Certificate %U", (unsigned long long)now);
+    sds cn = sdscatfmt(sdsempty(), "myMPD Server Certificate %I", (int64_t)now);
 
     X509_NAME *name = X509_REQ_get_subject_name(req);
     X509_NAME_add_entry_by_txt(name, "C",  MBSTRING_ASC, (unsigned char *)"DE", -1, -1, 0);
@@ -469,7 +552,7 @@ static X509_REQ *generate_request(EVP_PKEY *pkey) {
     FREE_SDS(cn);
 
     if (!X509_REQ_sign(req, pkey, EVP_sha256())) {
-        MYMPD_LOG_ERROR("Error signing request");
+        MYMPD_LOG_ERROR(NULL, "Error signing request");
         X509_REQ_free(req);
         return NULL;
     }
@@ -486,7 +569,7 @@ static X509_REQ *generate_request(EVP_PKEY *pkey) {
 static void add_extension(X509V3_CTX *ctx, X509 *cert, int nid, const char *value) {
     X509_EXTENSION *ex = X509V3_EXT_conf_nid(NULL, ctx, nid, value);
     if (!ex) {
-        MYMPD_LOG_ERROR("Error adding extension with value: %s", value);
+        MYMPD_LOG_ERROR(NULL, "Error adding extension with value: %s", value);
         return;
     }
     X509_add_ext(cert, ex, -1);
@@ -504,7 +587,7 @@ static void add_extension(X509V3_CTX *ctx, X509 *cert, int nid, const char *valu
 static X509 *sign_certificate_request(EVP_PKEY *ca_key, X509 *ca_cert, X509_REQ *req, sds san) {
     X509 *cert = X509_new();
     if (!cert) {
-        MYMPD_LOG_ERROR("Unable to create X509 structure");
+        MYMPD_LOG_ERROR(NULL, "Unable to create X509 structure");
         return NULL;
     }
 
@@ -519,8 +602,8 @@ static X509 *sign_certificate_request(EVP_PKEY *ca_key, X509 *ca_cert, X509_REQ
 
     /* This certificate is valid from now until one year from now. */
     int lifetime = CERT_LIFETIME * 24 * 60 * 60;
-    X509_gmtime_adj(X509_get_notBefore(cert), 0);
-    X509_gmtime_adj(X509_get_notAfter(cert), lifetime);
+    X509_gmtime_adj(X509_getm_notBefore(cert), 0);
+    X509_gmtime_adj(X509_getm_notAfter(cert), lifetime);
 
     /* Get the request's subject and just use it (we don't bother checking it since we generated
      * it ourself). Also take the request's public key. */
@@ -549,31 +632,50 @@ static X509 *sign_certificate_request(EVP_PKEY *ca_key, X509 *ca_cert, X509_REQ
 
 /**
  * Generates a private/public key pair
- * @param rsa_key_bits number of bits for the key
+ * @param key_bits number of bits for the key
+ * @param key_type key type: EVP_PKEY_RSA or EVP_PKEY_EC
  * @return newly allocated key or NULL on error
  */
-static EVP_PKEY *generate_keypair(int rsa_key_bits) {
-    EVP_PKEY_CTX *ctx;
-    EVP_PKEY *pkey = NULL;
-
-    ctx = EVP_PKEY_CTX_new_id(EVP_PKEY_RSA, NULL);
-    if (!ctx) {
-        return NULL;
-    }
-    if (EVP_PKEY_keygen_init(ctx) <= 0) {
+static EVP_PKEY *generate_keypair(int key_type, unsigned int key_bits) {
+    #ifdef OPENSSL_VERSION_MAJOR
+        if (key_type == EVP_PKEY_RSA) {
+            return EVP_RSA_gen(key_bits);
+        }
+        if (key_type == EVP_PKEY_EC) {
+            switch (key_bits) {
+                case 256: return EVP_EC_gen("P-256");
+                case 384: return EVP_EC_gen("P-384");
+                default:
+                    MYMPD_LOG_ERROR(NULL, "Invalid private key length");
+                    return NULL;
+            }
+        }
+    #else
+        // Support older versions of OpenSSL.
+        // Ignore key type, always use rsa keys.
+        (void)key_type;
+        EVP_PKEY *pkey = NULL;
+        EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new_id(EVP_PKEY_RSA, NULL);
+        if (!ctx) {
+            return NULL;
+        }
+        if (EVP_PKEY_keygen_init(ctx) <= 0) {
+            EVP_PKEY_CTX_free(ctx);
+            return NULL;
+        }
+        if (EVP_PKEY_CTX_set_rsa_keygen_bits(ctx, key_bits) <= 0) {
+            EVP_PKEY_CTX_free(ctx);
+            return NULL;
+        }
+        if (EVP_PKEY_keygen(ctx, &pkey) <= 0) {
+            EVP_PKEY_CTX_free(ctx);
+            return NULL;
+        }
         EVP_PKEY_CTX_free(ctx);
-        return NULL;
-    }
-    if (EVP_PKEY_CTX_set_rsa_keygen_bits(ctx, rsa_key_bits) <= 0) {
-        EVP_PKEY_CTX_free(ctx);
-        return NULL;
-    }
-    if (EVP_PKEY_keygen(ctx, &pkey) <= 0) {
-        EVP_PKEY_CTX_free(ctx);
-        return NULL;
-    }
-    EVP_PKEY_CTX_free(ctx);
-    return pkey;
+        return pkey;
+    #endif
+    MYMPD_LOG_ERROR(NULL, "Invalid private key type");
+    return NULL;
 }
 
 /**
@@ -585,7 +687,7 @@ static X509 *generate_selfsigned_cert(EVP_PKEY *pkey) {
     //Allocate memory for the X509 structure.
     X509 *cert = X509_new();
     if (!cert) {
-        MYMPD_LOG_ERROR("Unable to create X509 structure");
+        MYMPD_LOG_ERROR(NULL, "Unable to create X509 structure");
         return NULL;
     }
 
@@ -594,15 +696,15 @@ static X509 *generate_selfsigned_cert(EVP_PKEY *pkey) {
 
     //Set the serial number
     if (generate_set_random_serial(cert) == false) {
-        MYMPD_LOG_ERROR("Error setting serial for certificate");
-        X509_free(cert);    
+        MYMPD_LOG_ERROR(NULL, "Error setting serial for certificate");
+        X509_free(cert);
         return NULL;
     }
 
     //This certificate is valid from now until ten years from now.
     int lifetime = CA_LIFETIME * 24 * 60 * 60;
-    X509_gmtime_adj(X509_get_notBefore(cert), 0);
-    X509_gmtime_adj(X509_get_notAfter(cert), lifetime);
+    X509_gmtime_adj(X509_getm_notBefore(cert), 0);
+    X509_gmtime_adj(X509_getm_notAfter(cert), lifetime);
 
     //Set the public key for our certificate.
     X509_set_pubkey(cert, pkey);
@@ -612,7 +714,7 @@ static X509 *generate_selfsigned_cert(EVP_PKEY *pkey) {
 
     //Set the DN
     time_t now = time(NULL);
-    sds cn = sdscatfmt(sdsempty(), "myMPD CA %U", (unsigned long long)now);
+    sds cn = sdscatfmt(sdsempty(), "myMPD CA %I", (int64_t)now);
     X509_NAME_add_entry_by_txt(name, "C",  MBSTRING_ASC, (unsigned char *)"DE", -1, -1, 0);
     X509_NAME_add_entry_by_txt(name, "O",  MBSTRING_ASC, (unsigned char *)"myMPD", -1, -1, 0);
     X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC, (unsigned char *)cn, -1, -1, 0);
@@ -630,7 +732,7 @@ static X509 *generate_selfsigned_cert(EVP_PKEY *pkey) {
 
     //Self sign the certificate with our key.
     if (!X509_sign(cert, pkey, EVP_sha256())) {
-        MYMPD_LOG_ERROR("Error signing certificate");
+        MYMPD_LOG_ERROR(NULL, "Error signing certificate");
         X509_free(cert);
         return NULL;
     }
@@ -655,7 +757,7 @@ static bool write_to_disk(sds key_file, EVP_PKEY *pkey, sds cert_file, X509 *cer
         return false;
     }
     bool write_rc = PEM_write_PrivateKey(fp, pkey, NULL, NULL, 0, NULL, NULL) == 0 ? false : true;
-    bool rc = rename_tmp_file(fp, tmp_file, key_file, write_rc);
+    bool rc = rename_tmp_file(fp, tmp_file, write_rc);
     if (rc == false) {
         FREE_SDS(tmp_file);
         return rc;
@@ -670,7 +772,7 @@ static bool write_to_disk(sds key_file, EVP_PKEY *pkey, sds cert_file, X509 *cer
         return false;
     }
     write_rc = PEM_write_X509(fp, cert) == 0 ? false : true;
-    rc = rename_tmp_file(fp, tmp_file, cert_file, write_rc);
+    rc = rename_tmp_file(fp, tmp_file, write_rc);
     FREE_SDS(tmp_file);
     return rc;
 }
