@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -17,6 +17,6 @@ enum handle_options_rc {
     OPTIONS_RC_EXIT = 1
 };
 
-int handle_options(struct t_config *config, int argc, char **argv);
+enum handle_options_rc handle_options(struct t_config *config, int argc, char **argv);
 
 #endif
