@@ -1,12 +1,13 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
 #include "compile_time.h"
 #include "src/lib/list.h"
 
+#include "dist/utf8/utf8.h"
 #include "src/lib/filehandler.h"
 #include "src/lib/log.h"
 #include "src/lib/mem.h"
@@ -25,6 +26,39 @@ struct t_list *list_new(void) {
     return l;
 }
 
+/**
+ * Duplicates a list.
+ * Leaves user_data pointer in place.
+ * @param l list to duplicate
+ * @return duplicated list or NULL on error
+ */
+struct t_list *list_dup(struct t_list *l) {
+    struct t_list *new = list_new();
+    if (list_append(new, l) == false) {
+        list_free(new);
+        return NULL;
+    }
+    return new;
+}
+
+/**
+ * Appends a list to another list
+ * Leaves user_data pointer in place.
+ * @param dst list to that was append
+ * @param src list that was append
+ * @return duplicated list or NULL on error
+ */
+bool list_append(struct t_list *dst, struct t_list *src) {
+    struct t_list_node *current = src->head;
+    while (current != NULL) {
+        if (list_push(dst, current->key, current->value_i, current->value_p, current->user_data) == false) {
+            return false;
+        }
+        current = current->next;
+    }
+    return true;
+}
+
 /**
  * Inits a already allocated list
  * @param l pointer to list
@@ -49,6 +83,9 @@ void list_clear(struct t_list *l) {
  * @return NULL
  */
 void *list_free(struct t_list *l) {
+    if (l == NULL) {
+        return NULL;
+    }
     list_clear_user_data(l, list_free_cb_ignore_user_data);
     FREE_PTR(l);
     return NULL;
@@ -100,6 +137,33 @@ void list_free_cb_sds_user_data(struct t_list_node *current) {
     FREE_SDS(current->user_data);
 }
 
+/**
+ * Callback function to free user_data of generic pointer.
+ * @param current list node
+ */
+void list_free_cb_ptr_user_data(struct t_list_node *current) {
+    FREE_PTR(current->user_data);
+}
+
+/**
+ * Gets a list node index by key.
+ * @param l list
+ * @param key key to get
+ * @return int index of the key, UINT_MAX if not found
+ */
+unsigned list_get_node_idx(const struct t_list *l, const char *key) {
+    struct t_list_node *current = l->head;
+    unsigned i = 0;
+    while (current != NULL) {
+        if (strcmp(current->key, key) == 0) {
+            return i;
+        }
+        current = current->next;
+        i++;
+    }
+    return UINT_MAX;
+}
+
 /**
  * Gets a list node by key.
  * @param l list
@@ -124,7 +188,7 @@ struct t_list_node *list_get_node(const struct t_list *l, const char *key) {
  * @param previous pointer to previous node
  * @return pointer to list node
  */
-struct t_list_node *list_node_prev_at(const struct t_list *l, long idx, struct t_list_node **previous) {
+struct t_list_node *list_node_prev_at(const struct t_list *l, unsigned idx, struct t_list_node **previous) {
     //if there's no data in the list, fail
     if (l->head == NULL) {
         return NULL;
@@ -147,7 +211,7 @@ struct t_list_node *list_node_prev_at(const struct t_list *l, long idx, struct t
  * @param idx node index to get
  * @return pointer to list node
  */
-struct t_list_node *list_node_at(const struct t_list *l, long idx) {
+struct t_list_node *list_node_at(const struct t_list *l, unsigned idx) {
     struct t_list_node *previous = NULL;
     return list_node_prev_at(l, idx, &previous);
 }
@@ -159,11 +223,9 @@ struct t_list_node *list_node_at(const struct t_list *l, long idx) {
  * @param to to pos
  * @return true on success, else false
  */
-bool list_move_item_pos(struct t_list *l, long from, long to) {
+bool list_move_item_pos(struct t_list *l, unsigned from, unsigned to) {
     if (from >= l->length ||
-        to >= l->length ||
-        from < 0 ||
-        to < 0)
+        to >= l->length)
     {
         return false;
     }
@@ -171,7 +233,9 @@ bool list_move_item_pos(struct t_list *l, long from, long to) {
         return true;
     }
 
-    bool after = from < to ? true : false;
+    bool after = from < to
+        ? true
+        : false;
 
     //get new position
     struct t_list_node *previous = NULL;
@@ -220,7 +284,7 @@ bool list_swap_item(struct t_list_node *n1, struct t_list_node *n2) {
     }
 
     sds key = n2->key;
-    long long value_i = n2->value_i;
+    int64_t value_i = n2->value_i;
     sds value_p = n2->value_p;
     void *user_data = n2->user_data;
 
@@ -248,7 +312,7 @@ bool list_shuffle(struct t_list *l) {
     }
     struct t_list_node *current = l->head;
     while (current != NULL) {
-        long pos = randrange(0, l->length - 1);
+        unsigned pos = randrange(0, l->length);
         list_swap_item(current, list_node_at(l, pos));
         current = current->next;
     }
@@ -259,12 +323,12 @@ bool list_shuffle(struct t_list *l) {
  * Creates and appends a node to the end of the list.
  * @param l list
  * @param key key value
- * @param value_i long long value
+ * @param value_i int64_t value
  * @param value_p sds value
  * @param user_data pointer to user_data
  * @return true on success, else false
  */
-bool list_push(struct t_list *l, const char *key, long long value_i,
+bool list_push(struct t_list *l, const char *key, int64_t value_i,
         const char *value_p, void *user_data)
 {
     size_t value_len = value_p == NULL ? 0 : strlen(value_p);
@@ -277,13 +341,13 @@ bool list_push(struct t_list *l, const char *key, long long value_i,
  * @param l list
  * @param key key value
  * @param key_len key length
- * @param value_i long long value
+ * @param value_i int64_t value
  * @param value_p sds value
  * @param value_len value_p length
  * @param user_data pointer to user_data
  * @return true on success, else false
  */
-bool list_push_len(struct t_list *l, const char *key, size_t key_len, long long value_i,
+bool list_push_len(struct t_list *l, const char *key, size_t key_len, int64_t value_i,
         const char *value_p, size_t value_len, void *user_data)
 {
     //create new node
@@ -312,12 +376,12 @@ bool list_push_len(struct t_list *l, const char *key, size_t key_len, long long
  * Creates and inserts a node at the beginning of the list.
  * @param l list
  * @param key key value
- * @param value_i long long value
+ * @param value_i int64_t value
  * @param value_p sds value
  * @param user_data pointer to user_data
  * @return true on success, else false
  */
-bool list_insert(struct t_list *l, const char *key, long long value_i,
+bool list_insert(struct t_list *l, const char *key, int64_t value_i,
         const char *value_p, void *user_data)
 {
     //create new node
@@ -344,16 +408,39 @@ bool list_insert(struct t_list *l, const char *key, long long value_i,
  * @param l list
  * @param idx index of node to change
  * @param key new key value
- * @param value_i new long long value
+ * @param value_i new int64_t value
  * @param value_p new sds value
  * @param user_data new user_data pointer
  * @return true on success, else false
  */
-bool list_replace(struct t_list *l, long idx, const char *key, long long value_i,
+bool list_replace(struct t_list *l, unsigned idx, const char *key, int64_t value_i,
         const char *value_p, void *user_data)
 {
-    return list_replace_user_data(l, idx, key, value_i, value_p,
-        user_data, list_free_cb_ignore_user_data);
+    size_t value_len = value_p == NULL
+        ? 0
+        : strlen(value_p);
+    return list_replace_len_user_data(l, idx, key, strlen(key), value_i,
+        value_p, value_len, user_data, list_free_cb_ignore_user_data);
+}
+
+/**
+ * Replaces a list nodes values at pos.
+ * Ignores the old user_data pointer.
+ * @param l list
+ * @param idx index of node to change
+ * @param key new key value
+ * @param key_len new key value length
+ * @param value_i new int64_t value
+ * @param value_p new sds value
+ * @param value_len new sds value len
+ * @param user_data new user_data pointer
+ * @return true on success, else false
+ */
+bool list_replace_len(struct t_list *l, unsigned idx, const char *key, size_t key_len, int64_t value_i,
+        const char *value_p, size_t value_len, void *user_data)
+{
+    return list_replace_len_user_data(l, idx, key, key_len, value_i,
+        value_p, value_len, user_data, list_free_cb_ignore_user_data);
 }
 
 /**
@@ -362,24 +449,45 @@ bool list_replace(struct t_list *l, long idx, const char *key, long long value_i
  * @param l list
  * @param idx index of node to change
  * @param key new key value
- * @param value_i new long long value
+ * @param value_i new int64_t value
  * @param value_p new sds value
  * @param user_data new user_data pointer
  * @param free_cb callback function to free old user_data pointer
  * @return true on success, else false
  */
-bool list_replace_user_data(struct t_list *l, long idx, const char *key, long long value_i,
+bool list_replace_user_data(struct t_list *l, unsigned idx, const char *key, int64_t value_i,
         const char *value_p, void *user_data, user_data_callback free_cb)
+{
+    return list_replace_len_user_data(l, idx, key, strlen(key), value_i,
+        value_p, strlen(value_p), user_data, free_cb);
+}
+
+/**
+ * Replaces a list nodes values at pos.
+ * Frees the old user_data pointer.
+ * @param l list
+ * @param idx index of node to change
+ * @param key new key value
+ * @param key_len new key value length
+ * @param value_i new int64_t value
+ * @param value_p new sds value
+ * @param value_len new sds value len
+ * @param user_data new user_data pointer
+ * @param free_cb callback function to free old user_data pointer
+ * @return true on success, else false
+ */
+bool list_replace_len_user_data(struct t_list *l, unsigned idx, const char *key, size_t key_len, int64_t value_i,
+        const char *value_p, size_t value_len, void *user_data, user_data_callback free_cb)
 {
     if (idx >= l->length) {
         return false;
     }
     struct t_list_node *current = list_node_at(l, idx);
 
-    current->key = sds_replace(current->key, key);
+    current->key = sds_replacelen(current->key, key, key_len);
     current->value_i = value_i;
     if (value_p != NULL) {
-        current->value_p = sds_replace(current->value_p, value_p);
+        current->value_p = sds_replacelen(current->value_p, value_p, value_len);
     }
     else if (current->value_p != NULL) {
         FREE_SDS(current->value_p);
@@ -434,7 +542,7 @@ void *list_node_free_user_data(struct t_list_node *n, user_data_callback free_cb
  * @param idx node index to free
  * @return true on success, else false
  */
-bool list_remove_node(struct t_list *l, long idx) {
+bool list_remove_node(struct t_list *l, unsigned idx) {
     return list_remove_node_user_data(l, idx, list_free_cb_ignore_user_data);
 }
 
@@ -445,7 +553,7 @@ bool list_remove_node(struct t_list *l, long idx) {
  * @param free_cb callback function to free user_data pointer
  * @return true on success, else false
  */
-bool list_remove_node_user_data(struct t_list *l, long idx, user_data_callback free_cb) {
+bool list_remove_node_user_data(struct t_list *l, unsigned idx, user_data_callback free_cb) {
     struct t_list_node *extracted = list_node_extract(l, idx);
     if (extracted == NULL) {
         return false;
@@ -454,6 +562,36 @@ bool list_remove_node_user_data(struct t_list *l, long idx, user_data_callback f
     return true;
 }
 
+/**
+ * Removes the node with key
+ * Ignores user_data pointer
+ * @param l list
+ * @param key key
+ * @return bool true on success, else false
+ */
+bool list_remove_node_by_key(struct t_list *l, const char *key) {
+    return list_remove_node_by_key_user_data(l, key, NULL);
+}
+
+/**
+ * Removes the node with key
+ * @param l list
+ * @param key key
+ * @return bool true on success, else false
+ */
+bool list_remove_node_by_key_user_data(struct t_list *l, const char *key, user_data_callback free_cb) {
+    struct t_list_node *current = l->head;
+    unsigned i = 0;
+    while (current != NULL) {
+        if (strcmp(current->key, key) == 0) {
+            return list_remove_node_user_data(l, i, free_cb);
+        }
+        i++;
+        current = current->next;
+    }
+    return false;
+}
+
 /**
  * Removes the first node from the list and returns it.
  * This is only a shortcut for list_node_extract.
@@ -470,7 +608,7 @@ struct t_list_node *list_shift_first(struct t_list *l) {
  * @param idx node index to remove
  * @return pointer to list node
  */
-struct t_list_node *list_node_extract(struct t_list *l, long idx) {
+struct t_list_node *list_node_extract(struct t_list *l, unsigned idx) {
     if (l->head == NULL ||
         idx >= l->length) {
         return NULL;
@@ -524,9 +662,9 @@ bool list_write_to_disk(sds filepath, struct t_list *l, list_node_to_line_callba
     sds buffer = sdsempty();
     bool write_rc = true;
     while (current != NULL) {
-        buffer = node_to_line_cb(buffer, current);
+        buffer = node_to_line_cb(buffer, current, true);
         if (fputs(buffer, fp) == EOF) {
-            MYMPD_LOG_ERROR("Could not write data to file");
+            MYMPD_LOG_ERROR(NULL, "Could not write data to file");
             write_rc = false;
             break;
         }
@@ -534,7 +672,146 @@ bool list_write_to_disk(sds filepath, struct t_list *l, list_node_to_line_callba
         current = current->next;
     }
     FREE_SDS(buffer);
-    bool rc = rename_tmp_file(fp, tmp_file, filepath, write_rc);
+    bool rc = rename_tmp_file(fp, tmp_file, write_rc);
     FREE_SDS(tmp_file);
     return rc;
 }
+
+/**
+ * Crops the list to the defined length
+ * @param l pointer to list to crop
+ * @param length max length
+ * @param free_cb callback function to free user_data pointer
+ */
+void list_crop(struct t_list *l, unsigned length, user_data_callback free_cb) {
+    if (l->length <= length) {
+        return;
+    }
+    if (length == 0) {
+        list_clear(l);
+        return;
+    }
+    unsigned idx = length - 1;
+    struct t_list_node *last = list_node_at(l, idx);
+    struct t_list_node *current = last->next;
+    while (current != NULL) {
+        struct t_list_node *next = current->next;
+        list_node_free_user_data(current, free_cb);
+        current = next;
+    }
+    l->length = length;
+    last->next = NULL;
+    l->tail = last;
+}
+
+/**
+ * Internal compare function to sort by value_i
+ * @param current current list node
+ * @param next next list node
+ * @param direction sort direction
+ * @return true if current is greater than next
+ */
+static bool list_sort_cmp_value_i(struct t_list_node *current, struct t_list_node *next, enum list_sort_direction direction) {
+    if ((direction == LIST_SORT_ASC && current->value_i > next->value_i) ||
+        (direction == LIST_SORT_DESC && current->value_i < next->value_i))
+    {
+        return true;
+    }
+    return false;
+}
+
+/**
+ * Internal compare function to sort by value_p
+ * @param current current list node
+ * @param next next list node
+ * @param direction sort direction
+ * @return true if current is greater than next
+ */
+static bool list_sort_cmp_value_p(struct t_list_node *current, struct t_list_node *next, enum list_sort_direction direction) {
+    int result = utf8casecmp(current->value_p, next->value_p);
+    if ((direction == LIST_SORT_ASC && result > 0) ||
+        (direction == LIST_SORT_DESC && result < 0))
+    {
+        return true;
+    }
+    return false;
+}
+
+/**
+ * Internal compare function to sort by key
+ * @param current current list node
+ * @param next next list node
+ * @param direction sort direction
+ * @return true if current is greater than next
+ */
+static bool list_sort_cmp_key(struct t_list_node *current, struct t_list_node *next, enum list_sort_direction direction) {
+    int result = utf8casecmp(current->key, next->key);
+    if ((direction == LIST_SORT_ASC && result > 0) ||
+        (direction == LIST_SORT_DESC && result < 0))
+    {
+        return true;
+    }
+    return false;
+}
+
+/**
+ * The list sorting function.
+ * Note that the sort is not very efficient, use it only for short lists.
+ * @param l pointer to list to sort
+ * @param direction sort direction
+ * @param sort_cb compare function
+ * @return true on success, else false
+ */
+bool list_sort_by_callback(struct t_list *l, enum list_sort_direction direction, list_sort_callback sort_cb) {
+    int swapped;
+    struct t_list_node *ptr1;
+    struct t_list_node *lptr = NULL;
+
+    if (l->head == NULL) {
+        return false;
+    }
+
+    do {
+        swapped = 0;
+        ptr1 = l->head;
+        while (ptr1->next != lptr) {
+            if (sort_cb(ptr1, ptr1->next, direction) == true) {
+                list_swap_item(ptr1, ptr1->next);
+                swapped = 1;
+            }
+            ptr1 = ptr1->next;
+        }
+        lptr = ptr1;
+    } while (swapped);
+    return true;
+}
+
+/**
+ * Sorts the list by value_i
+ * @param l pointer to list to sort
+ * @param direction sort direction
+ * @return true on success, else false
+ */
+bool list_sort_by_value_i(struct t_list *l, enum list_sort_direction direction) {
+    return list_sort_by_callback(l, direction, list_sort_cmp_value_i);
+}
+
+/**
+ * Sorts the list by value_p
+ * @param l pointer to list to sort
+ * @param direction sort direction
+ * @return true on success, else false
+ */
+bool list_sort_by_value_p(struct t_list *l, enum list_sort_direction direction) {
+    return list_sort_by_callback(l, direction, list_sort_cmp_value_p);
+}
+
+/**
+ * Sorts the list by key
+ * @param l pointer to list to sort
+ * @param direction sort direction
+ * @return true on success, else false
+ */
+bool list_sort_by_key(struct t_list *l, enum list_sort_direction direction) {
+    return list_sort_by_callback(l, direction, list_sort_cmp_key);
+}
