@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -9,12 +9,12 @@
 
 #include "src/lib/mympd_state.h"
 
-bool mpd_client_connect(struct t_partition_state *partition_state, bool detect_feat);
+bool mpd_client_connect(struct t_partition_state *partition_state);
 bool mpd_client_set_keepalive(struct t_partition_state *partition_state);
 bool mpd_client_set_timeout(struct t_partition_state *partition_state);
 bool mpd_client_set_binarylimit(struct t_partition_state *partition_state);
 bool mpd_client_set_connection_options(struct t_partition_state *partition_state);
-void mpd_client_disconnect(struct t_partition_state *partition_state, enum mpd_conn_states new_conn_state);
-void mpd_client_disconnect_silent(struct t_partition_state *partition_state, enum mpd_conn_states new_conn_state);
-void mpd_client_disconnect_all(struct t_mympd_state *mympd_state, enum mpd_conn_states new_conn_state);
+void mpd_client_disconnect(struct t_partition_state *partition_state);
+void mpd_client_disconnect_silent(struct t_partition_state *partition_state);
+void mpd_client_disconnect_all(struct t_mympd_state *mympd_state);
 #endif
