@@ -1,20 +1,24 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
 #include "compile_time.h"
-#include "tags.h"
+#include "src/mpd_client/tags.h"
 
-#include "dist/libmpdclient/src/isong.h"
+#include "dist/libmympdclient/src/isong.h"
+#include "src/lib/cache_rax_album.h"
+#include "src/lib/convert.h"
 #include "src/lib/jsonrpc.h"
 #include "src/lib/log.h"
 #include "src/lib/mem.h"
 #include "src/lib/sds_extras.h"
 #include "src/lib/utility.h"
 #include "src/mpd_client/errorhandler.h"
+#include "src/mpd_client/shortcuts.h"
 
+#include <inttypes.h>
 #include <string.h>
 
 /**
@@ -30,6 +34,25 @@ static sds get_tag_values(const struct mpd_song *song, enum mpd_tag_type tag,
  * Public functions
  */
 
+/**
+ * Returns the mpd database last modification time
+ * @param partition_state pointer to partition specific states
+ * @return last modification time
+ */
+time_t mpd_client_get_db_mtime(struct t_partition_state *partition_state) {
+    time_t mtime = 0;
+    struct mpd_stats *stats = mpd_run_stats(partition_state->conn);
+    if (stats != NULL) {
+        mtime = (time_t)mpd_stats_get_db_update_time(stats);
+        mpd_stats_free(stats);
+    }
+    mpd_response_finish(partition_state->conn);
+    if (mympd_check_error_and_recover(partition_state, NULL, "mpd_run_stats") == false) {
+        mtime = 0;
+    }
+    return mtime;
+}
+
 /**
  * Adds a tag value to the album if value does not already exists
  * @param song pointer to a mpd_song struct
@@ -73,7 +96,7 @@ bool mympd_mpd_song_add_tag_dedup(struct mpd_song *song,
 
         tag->value = strdup(value);
         if (tag->value == NULL) {
-        FREE_PTR(tag);
+            FREE_PTR(tag);
             return false;
         }
 
@@ -110,7 +133,22 @@ bool is_multivalue_tag(enum mpd_tag_type tag) {
 }
 
 /**
- * Maps a tag to its sort tag pedant and checks if the sort tag is enabled.
+ * Checks if tag is a numeric tag
+ * @param tag mpd tag type
+ * @return true if it is a numeric tag, else false
+ */
+bool is_numeric_tag(enum mpd_tag_type tag) {
+    switch(tag) {
+        case MPD_TAG_DISC:
+        case MPD_TAG_TRACK:
+            return true;
+        default:
+            return false;
+    }
+}
+
+/**
+ * Maps a tag to its sort tag pendant and checks if the sort tag is enabled.
  * @param tag mpd tag type
  * @param available_tags pointer to enabled tags
  * @return sort tag if exists, else the original tag
@@ -136,7 +174,38 @@ enum mpd_tag_type get_sort_tag(enum mpd_tag_type tag, const struct t_tags *avail
         default:
             return tag;
     }
-    return mpd_client_tag_exists(available_tags, sort_tag) == true ? sort_tag : tag;
+    return mpd_client_tag_exists(available_tags, sort_tag) == true
+        ? sort_tag
+        : tag;
+}
+
+/**
+ * Gets an alphanumeric string for sorting
+ * @param key already allocated sds string to append
+ * @param sort_by enum sort_by
+ * @param sort_tag mpd tag to sort by
+ * @param song pointer to mpd song
+ * @return pointer to key
+ */
+sds get_sort_key(sds key, enum sort_by_type sort_by, enum mpd_tag_type sort_tag, const struct mpd_song *song) {
+    if (sort_by == SORT_BY_LAST_MODIFIED) {
+        key = mpd_client_get_value_padded((int64_t)mpd_song_get_last_modified(song), key);
+    }
+    else if (sort_by == SORT_BY_ADDED) {
+        key = mpd_client_get_value_padded((int64_t)mpd_song_get_added(song), key);
+    }
+    else if (is_numeric_tag(sort_tag) == true) {
+        key = mpd_client_get_tag_value_padded(song, sort_tag, '0', PADDING_LENGTH, key);
+    }
+    else if (sort_tag > MPD_TAG_UNKNOWN) {
+        key = mpd_client_get_tag_value_string(song, sort_tag, key);
+        if (sdslen(key) == 0) {
+            key = sdscatlen(key, "zzzzzzzzzz", 10);
+        }
+    }
+    key = sdscatfmt(key, "::%s", mpd_song_get_uri(song));
+    sds_utf8_tolower(key);
+    return key;
 }
 
 /**
@@ -144,9 +213,9 @@ enum mpd_tag_type get_sort_tag(enum mpd_tag_type tag, const struct t_tags *avail
  * @param partition_state pointer to partition specific states
  */
 bool disable_all_mpd_tags(struct t_partition_state *partition_state) {
-    MYMPD_LOG_DEBUG("\"%s\": Disabling all mpd tag types", partition_state->name);
-    bool rc = mpd_run_clear_tag_types(partition_state->conn);
-    return mympd_check_rc_error_and_recover(partition_state, rc, "mpd_run_clear_tag_types");
+    MYMPD_LOG_DEBUG(partition_state->name, "Disabling all mpd tag types");
+    mpd_run_clear_tag_types(partition_state->conn);
+    return mympd_check_error_and_recover(partition_state, NULL, "mpd_run_clear_tag_types");
 }
 
 /**
@@ -154,37 +223,114 @@ bool disable_all_mpd_tags(struct t_partition_state *partition_state) {
  * @param partition_state pointer to partition specific states
  */
 bool enable_all_mpd_tags(struct t_partition_state *partition_state) {
-    MYMPD_LOG_DEBUG("\"%s\": Enabling all mpd tag types", partition_state->name);
-    bool rc = mpd_run_all_tag_types(partition_state->conn);
-    return mympd_check_rc_error_and_recover(partition_state, rc, "mpd_run_all_tag_types");
+    MYMPD_LOG_DEBUG(partition_state->name, "Enabling all mpd tag types");
+    mpd_run_all_tag_types(partition_state->conn);
+    return mympd_check_error_and_recover(partition_state, NULL, "mpd_run_all_tag_types");
+}
+
+/**
+ * Helper function to print athe tags of a t_fields struct as json array
+ * @param buffer already allocated sds string to append the response
+ * @param tagsname key for the json array
+ * @param tags tags to print
+ * @return pointer to buffer
+ */
+sds print_tags_array(sds buffer, const char *tagsname, const struct t_tags *tags) {
+    buffer = sdscatfmt(buffer, "\"%s\": [", tagsname);
+    for (unsigned i = 0; i < tags->len; i++) {
+        if (i > 0) {
+            buffer = sdscatlen(buffer, ",", 1);
+        }
+        const char *tagname = mpd_tag_name(tags->tags[i]);
+        buffer = sds_catjson(buffer, tagname, strlen(tagname));
+    }
+    buffer = sdscatlen(buffer, "]", 1);
+    return buffer;
 }
 
 /**
  * Enables specific mpd tags
  * @param partition_state pointer to partition specific states
- * @param enable_tags pointer to t_tags struct
+ * @param enable_tags pointer to t_fields struct
  */
 bool enable_mpd_tags(struct t_partition_state *partition_state, const struct t_tags *enable_tags) {
-    if (partition_state->mpd_state->feat_tags == false) {
+    if (partition_state->mpd_state->feat.tags == false) {
         return true;
     }
-    MYMPD_LOG_INFO("\"%s\": Setting interesting mpd tag types", partition_state->name);
+    MYMPD_LOG_INFO(partition_state->name, "Setting interesting mpd tag types");
     if (mpd_command_list_begin(partition_state->conn, false)) {
-        bool rc = mpd_send_clear_tag_types(partition_state->conn);
-        if (rc == false) {
-            MYMPD_LOG_ERROR("Error adding command to command list mpd_send_clear_tag_types");
+        if (mpd_send_clear_tag_types(partition_state->conn) == false) {
+            mympd_set_mpd_failure(partition_state, "Error adding command to command list mpd_send_clear_tag_types");
         }
         if (enable_tags->len > 0) {
-            rc = mpd_send_enable_tag_types(partition_state->conn, enable_tags->tags, (unsigned)enable_tags->len);
-            if (rc == false) {
-                MYMPD_LOG_ERROR("Error adding command to command list mpd_send_enable_tag_types");
+            if (mpd_send_enable_tag_types(partition_state->conn, enable_tags->tags, (unsigned)enable_tags->len) == false) {
+                mympd_set_mpd_failure(partition_state, "Error adding command to command list mpd_send_enable_tag_types");
             }
         }
-        if (mpd_command_list_end(partition_state->conn)) {
-            mpd_response_finish(partition_state->conn);
+        else {
+            MYMPD_LOG_WARN(partition_state->name, "No mpd tags are enabled");
+        }
+        mpd_client_command_list_end_check(partition_state);
+    }
+    mpd_response_finish(partition_state->conn);
+    return mympd_check_error_and_recover(partition_state, NULL, "mpd_send_enable_tag_types");
+}
+
+/**
+ * Get's a tag value from mpd song and converts it to int
+ * @param song mpd song struct
+ * @param tag mdp tag type
+ * @return parsed tag value
+ */
+int mpd_client_get_tag_value_int(const struct mpd_song *song, enum mpd_tag_type tag) {
+    const char *value = mpd_song_get_tag(song, tag, 0);
+    if (value == NULL) {
+        return 0;
+    }
+    int int_value;
+    enum str2int_errno rc = str2int(&int_value, value);
+    return rc == STR2INT_SUCCESS
+        ? int_value
+        : 0;
+}
+
+/**
+ * Get's a tag value from mpd song and pads it
+ * @param song mpd song struct
+ * @param tag mpd tag type
+ * @param pad padding char
+ * @param len length to pad
+ * @param tag_values already allocated sds string to append
+ * @return sds new sds pointer to tag_values
+ */
+sds mpd_client_get_tag_value_padded(const struct mpd_song *song, enum mpd_tag_type tag, const char pad, size_t len, sds tag_values) {
+    const char *value = mpd_song_get_tag(song, tag, 0);
+    size_t value_len = value == NULL
+        ? 0
+        : strlen(value);
+    if (value_len < len) {
+        len = len - value_len;
+        for (size_t i = 0; i < len; i++) {
+            tag_values = sdscatfmt(tag_values, "%c", pad);
         }
     }
-    return mympd_check_error_and_recover(partition_state);
+    if (value != NULL) {
+        tag_values = sdscatlen(tag_values, value, value_len);
+    }
+    return tag_values;
+}
+
+/**
+ * Prints a zero padded value
+ * @param song mpd song struct
+ * @param tag mpd tag type
+ * @param pad padding char
+ * @param len length to pad
+ * @param tag_values already allocated sds string to append
+ * @return sds new sds pointer to tag_values
+ */
+sds mpd_client_get_value_padded(int64_t value, sds tag_values) {
+    return sdscatprintf(tag_values, "%020" PRId64, value);
 }
 
 /**
@@ -236,101 +382,82 @@ sds mpd_client_get_tag_values(const struct mpd_song *song, enum mpd_tag_type tag
             }
         }
         else {
-            //replace empty tag value(s) with dash
-            if (multi == true) {
-                tag_values = sdscatlen(tag_values, "[\"-\"]", 5);
-            }
-            else {
-                tag_values = sdscatlen(tag_values, "\"-\"", 3);
-            }
+            //set empty tag value(s)
+            tag_values = multi == true
+                ? sdscatlen(tag_values, "[]", 2)
+                : sdscatlen(tag_values, "\"\"", 2);
         }
     }
     return tag_values;
 }
 
 /**
- * Gets the tag values for a mpd song as json string
+ * Prints the tag values for a mpd song as json string
  * @param buffer already allocated sds string to append the values
- * @param partition_state pointer to partition specific states
- * @param tagcols pointer to t_tags struct (tags to retrieve)
+ * @param mpd_state pointer to mpd_state
+ * @param tagcols pointer to t_fields struct (tags to retrieve)
  * @param song pointer to a mpd_song struct to retrieve tags from
  * @return new sds pointer to buffer
  */
-sds get_song_tags(sds buffer, struct t_partition_state *partition_state, const struct t_tags *tagcols,
-                  const struct mpd_song *song)
+sds print_song_tags(sds buffer, const struct t_mpd_state *mpd_state, const struct t_tags *tagcols,
+        const struct mpd_song *song)
 {
-    if (partition_state->mpd_state->feat_tags == true) {
+    const char *uri = mpd_song_get_uri(song);
+    if (mpd_state->feat.tags == true) {
         for (unsigned tagnr = 0; tagnr < tagcols->len; ++tagnr) {
             buffer = sdscatfmt(buffer, "\"%s\":", mpd_tag_name(tagcols->tags[tagnr]));
             buffer = mpd_client_get_tag_values(song, tagcols->tags[tagnr], buffer);
             buffer = sdscatlen(buffer, ",", 1);
         }
+        if (is_streamuri(uri) == false) {
+            sds albumid = album_cache_get_key(sdsempty(), song, &mpd_state->config->albums);
+            buffer = tojson_sds(buffer, "AlbumId", albumid, true);
+            FREE_SDS(albumid);
+        }
     }
     else {
         buffer = sdscat(buffer, "\"Title\":");
         buffer = mpd_client_get_tag_values(song, MPD_TAG_TITLE, buffer);
         buffer = sdscatlen(buffer, ",", 1);
     }
-
     buffer = tojson_uint(buffer, "Duration", mpd_song_get_duration(song), true);
-    buffer = tojson_llong(buffer, "LastModified", (long long)mpd_song_get_last_modified(song), true);
-    buffer = tojson_char(buffer, "uri", mpd_song_get_uri(song), false);
+    buffer = tojson_time(buffer, "Last-Modified", mpd_song_get_last_modified(song), true);
+    if (mpd_state->feat.db_added == true) {
+        buffer = tojson_time(buffer, "Added", mpd_song_get_added(song), true);
+    }
+    buffer = tojson_char(buffer, "uri", uri, false);
     return buffer;
 }
 
 /**
- * Gets the same json string as get_song_tags but with empty values, title is set to the basefilename
- * @param buffer alread allocated sds string to append the values
- * @param partition_state pointer to partition specific states
+ * Prints the tag values for an album as json string
+ * @param buffer already allocated sds string to append the values
+ * @param mpd_state pointer to mpd_state
  * @param tagcols pointer to t_tags struct (tags to retrieve)
- * @param uri uri for printing empty tags
+ * @param album pointer to a mpd_song struct representing the album
  * @return new sds pointer to buffer
  */
-sds get_empty_song_tags(sds buffer, struct t_partition_state *partition_state, const struct t_tags *tagcols,
-                        const char *uri)
+sds print_album_tags(sds buffer, const struct t_mpd_state *mpd_state, const struct t_tags *tagcols,
+        const struct mpd_song *album)
 {
-    sds filename = sdsnew(uri);
-    basename_uri(filename);
-    if (partition_state->mpd_state->feat_tags == true) {
-        for (unsigned tagnr = 0; tagnr < tagcols->len; ++tagnr) {
-            const bool multi = is_multivalue_tag(tagcols->tags[tagnr]);
-            buffer = sdscatfmt(buffer, "\"%s\":", mpd_tag_name(tagcols->tags[tagnr]));
-            if (multi == true) {
-                buffer = sdscatlen(buffer, "[", 1);
-            }
-            if (tagcols->tags[tagnr] == MPD_TAG_TITLE) {
-                buffer = sds_catjson(buffer, filename, sdslen(filename));
-            }
-            else {
-                buffer = sdscatlen(buffer, "\"-\"", 3);
-            }
-            if (multi == true) {
-                buffer = sdscatlen(buffer, "]", 1);
-            }
-            buffer = sdscatlen(buffer, ",", 1);
-        }
-    }
-    else {
-        buffer = tojson_sds(buffer, "Title", filename, true);
-    }
-    buffer = tojson_long(buffer, "Duration", 0, true);
-    buffer = tojson_long(buffer, "LastModified", 0, true);
-    buffer = tojson_char(buffer, "uri", uri, false);
-    FREE_SDS(filename);
+    buffer = print_song_tags(buffer, mpd_state, tagcols, album);
+    buffer = sdscatlen(buffer, ",", 1);
+    buffer = tojson_uint(buffer, "Discs", album_get_discs(album), true);
+    buffer = tojson_uint(buffer, "SongCount", album_get_song_count(album), false);
     return buffer;
 }
 
 /**
  * Prints the audioformat as json object
  * @param buffer already allocated sds string to append the values
- * @param audioformat pointer to t_tags struct (tags to retrieve)
+ * @param audioformat pointer to t_fields struct (tags to retrieve)
  * @return new sds pointer to buffer
  */
 sds printAudioFormat(sds buffer, const struct mpd_audio_format *audioformat) {
     buffer = sdscat(buffer, "\"AudioFormat\":{");
     buffer = tojson_uint(buffer, "sampleRate", (audioformat ? audioformat->sample_rate : 0), true);
-    buffer = tojson_long(buffer, "bits", (audioformat ? audioformat->bits : 0), true);
-    buffer = tojson_long(buffer, "channels", (audioformat ? audioformat->channels : 0), false);
+    buffer = tojson_uint(buffer, "bits", (audioformat ? audioformat->bits : 0), true);
+    buffer = tojson_uint(buffer, "channels", (audioformat ? audioformat->channels : 0), false);
     buffer = sdscatlen(buffer, "}", 1);
     return buffer;
 }
@@ -340,7 +467,7 @@ sds printAudioFormat(sds buffer, const struct mpd_audio_format *audioformat) {
  * @param taglist comma separated tags to check
  * @param taglistname descriptive name of taglist
  * @param tagtypes pointer to t_tags struct to add tags from taglist
- * @param allowed_tag_types pointer to t_tags struct for allowed tags
+ * @param allowed_tag_types pointer to t_fields struct for allowed tags
  */
 void check_tags(sds taglist, const char *taglistname, struct t_tags *tagtypes,
                 const struct t_tags *allowed_tag_types)
@@ -352,7 +479,7 @@ void check_tags(sds taglist, const char *taglistname, struct t_tags *tagtypes,
         sdstrim(tokens[i], " ");
         enum mpd_tag_type tag = mpd_tag_name_iparse(tokens[i]);
         if (tag == MPD_TAG_UNKNOWN) {
-            MYMPD_LOG_WARN("Unknown tag %s", tokens[i]);
+            MYMPD_LOG_WARN(NULL, "Unknown tag %s", tokens[i]);
         }
         else {
             if (mpd_client_tag_exists(allowed_tag_types, tag) == true) {
@@ -360,12 +487,12 @@ void check_tags(sds taglist, const char *taglistname, struct t_tags *tagtypes,
                 tagtypes->tags[tagtypes->len++] = tag;
             }
             else {
-                MYMPD_LOG_DEBUG("Disabling tag %s", mpd_tag_name(tag));
+                MYMPD_LOG_DEBUG(NULL, "Disabling tag %s", mpd_tag_name(tag));
             }
         }
     }
     sdsfreesplitres(tokens, tokens_count);
-    MYMPD_LOG_NOTICE("%s", logline);
+    MYMPD_LOG_NOTICE(NULL, "%s", logline);
     FREE_SDS(logline);
 }
 
@@ -413,7 +540,8 @@ static sds get_tag_value_string(const struct mpd_song *song, enum mpd_tag_type t
 }
 
 /**
- * Appends a json string or array to tag_values
+ * Appends a json string or array to tag_values.
+ * Nothing is append if value is empty.
  * @param song pointer to mpd song struct
  * @param tag mpd tag type to get values for
  * @param tag_values already allocated sds string to append the values
