@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -10,5 +10,7 @@
 #include "src/lib/api.h"
 #include "src/lib/mympd_state.h"
 
-bool mpd_worker_start(struct t_mympd_state *mympd_state, struct t_work_request *request);
+bool mpd_worker_start(struct t_mympd_state *mympd_state, struct t_partition_state *partition_state,
+        struct t_work_request *request);
+
 #endif
