@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -9,6 +9,6 @@
 
 #include "dist/sds/sds.h"
 
-sds mympd_api_smartpls_get(sds workdir, sds buffer, long request_id, const char *playlist);
+sds mympd_api_smartpls_get(sds workdir, sds buffer, unsigned request_id, const char *playlist);
 
 #endif
