@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -9,5 +9,5 @@
 
 #include "src/lib/mympd_state.h"
 
-sds mympd_api_stats_get(struct t_partition_state *partition_state, sds buffer, long request_id);
+sds mympd_api_stats_get(struct t_partition_state *partition_state, sds buffer, unsigned request_id);
 #endif
