@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -19,7 +19,9 @@ enum albumart_sizes {
     ALBUMART_FULL
 };
 
+void webserver_send_albumart_redirect(struct mg_connection *nc, sds data);
 void webserver_send_albumart(struct mg_connection *nc, sds data, sds binary);
-bool request_handler_albumart(struct mg_connection *nc, struct mg_http_message *hm,
-    struct t_mg_user_data *mg_user_data, long long conn_id, enum albumart_sizes size);
+void request_handler_albumart_by_album_id(struct mg_http_message *hm, unsigned long conn_id, enum albumart_sizes size);
+bool request_handler_albumart_by_uri(struct mg_connection *nc, struct mg_http_message *hm,
+    struct t_mg_user_data *mg_user_data, unsigned long conn_id, enum albumart_sizes size);
 #endif
