@@ -1,6 +1,6 @@
 /*
  SPDX-License-Identifier: GPL-3.0-or-later
- myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
+ myMPD (c) 2018-2024 Juergen Mang <mail@jcgames.de>
  https://github.com/jcorporation/mympd
 */
 
@@ -12,5 +12,6 @@
 
 #include <stdbool.h>
 
-bool request_handler_tagart(struct mg_connection *nc, struct mg_http_message *hm, struct t_mg_user_data *mg_user_data);
+bool request_handler_tagart(struct mg_connection *nc, struct mg_http_message *hm,
+        struct t_mg_user_data *mg_user_data, unsigned long conn_id);
 #endif
